/**
 * Copyright (C) 2012  Dušan Vejnovič  <vaadin@dussan.org>
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 * http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */
package org.dussan.vaadin.dtable;

import java.text.Format;
import java.util.ArrayList;
import java.util.HashMap;
import java.util.List;
import java.util.Map;
import java.util.Set;

import org.apache.commons.lang3.ArrayUtils;
import org.dussan.vaadin.dtable.client.ui.VDTable;
import org.dussan.vaadin.dtable.component.DTableComponent;
import org.dussan.vaadin.dtable.config.TableConfig;
import org.dussan.vaadin.dtable.config.tablesection.alignments.ColumnAlign;
import org.dussan.vaadin.dtable.info.ManifestInfo;
import org.dussan.vaadin.dtable.processors.FilterProcessor;
import org.dussan.vaadin.dtable.processors.FormattingProcessor;
import org.dussan.vaadin.dtable.processors.OrderProcessor;
import org.dussan.vaadin.dtable.processors.SearchProcessor;
import org.dussan.vaadin.dtable.properties.filter.Filter;
import org.dussan.vaadin.dtable.properties.order.Order;
import org.dussan.vaadin.dtable.properties.search.Search;
import org.dussan.vaadin.dtable.translations.TranslatedWord;
import org.dussan.vaadin.dtable.windows.WindowsHelper;

import com.vaadin.terminal.PaintException;
import com.vaadin.terminal.PaintTarget;
import com.vaadin.ui.ClientWidget;
import com.vaadin.ui.Window.Notification;

@SuppressWarnings("serial")
@ClientWidget(VDTable.class)
public abstract class DTable extends DTableComponent {

	private boolean isTableInitialized = false;
	private boolean paintObjectsIsProcessedNormal = false;

	private int frozenAndVisibleTableColumnsLength = 0;
	private Map<String, Object> paintObjects = new HashMap<String, Object>();
	private Map<String, Object> archivedPaintObjects = new HashMap<String, Object>();
	private WindowsHelper windowsHelper = null;

	public DTable(String name) {
		// don't remove: it define dynamic widget's width
		setWidth("100%");
		getTableExporter().setBasename(name);
	}

	abstract public void exportProcessCompleted();

	abstract public long getAllRows(Filter[] filters, Filter[] quickSearch);

	abstract public Object[] getRowsData(int startRow, int rows,
			Order[] orders, Filter[] filters, Filter[] quickSearch);

	abstract public boolean prepareExportedData(String name, Order[] orders,
			Filter[] filters, Filter[] quickSearch);

	abstract public long getExportedAllRows(String name, Filter[] orders,
			Filter[] quickSearch);

	abstract public Object[] getExportedData(String name, int startRow,
			int rows, Order[] orders, Filter[] filters, Filter[] quickSearch);

	public static String getTitle() {
		if (ManifestInfo.getManifest() != null) {
			return ManifestInfo.getManifest().getMainAttributes()
					.getValue("Implementation-Title");
		}
		return null;
	}

	public static String getVersion() {
		if (ManifestInfo.getManifest() != null) {
			return ManifestInfo.getManifest().getMainAttributes()
					.getValue("Implementation-Version");
		}
		return null;
	}

	public static String getGitVersion() {
		if (ManifestInfo.getManifest() != null) {
			return ManifestInfo.getManifest().getMainAttributes()
					.getValue("Git-Version");
		}
		return null;
	}

	public static String getGwtQueryVersion() {
		if (ManifestInfo.getManifest() != null) {
			return ManifestInfo.getManifest().getMainAttributes()
					.getValue("GwtQuery-Version");
		}
		return null;
	}

	private void prepareTableQuery(Order[] orders, Filter[] filters,
			Filter[] quickSearch, long startRow, int rows) {
		boolean enabledAutoPrepareDataWithoutHiddenColumns = getTableConfig()
				.isEnabledAutoPrepareDataWithoutHiddenColumns();
		boolean enabledAutoPrepareDataWithoutRemovedColumns = getTableConfig()
				.isEnabledAutoPrepareDataWithoutRemovedColumns();
		int[] removedColumns = getTableProperties().getHiddenAndRemovedColumns(
				enabledAutoPrepareDataWithoutHiddenColumns,
				enabledAutoPrepareDataWithoutRemovedColumns);
		if (getTableConfig().isEnabledAutoPrepareData()) {
			getTableQuery()
					.unlockQuery()
					.lockQuery()
					.setProperties(orders, filters, quickSearch, removedColumns);
			if (startRow >= 0 && rows > 0) {
				getTableQuery().getQuery().addLimit(startRow, rows);
			}
		}
	}

	private boolean refreshTableHeader(int columnsLength, boolean fetchNewData,
			boolean hideRequest, boolean freezeRequest, boolean sortRequest) {
		return (columnsLength != this.frozenAndVisibleTableColumnsLength
				|| Boolean.FALSE.equals(fetchNewData)
				|| (hideRequest && Boolean.TRUE.equals(fetchNewData))
				|| (freezeRequest && getTableConfig().isEnabledFrozenTable())
				|| (sortRequest && (getTableConfig()
						.isEnabledOrderedFrozenTable() || getTableConfig()
						.isEnabledOrderedTable())) || getTableProperties()
				.getScreenWidth() == TableConfig.DEFAULT_TABLE_WIDTH);
	}

	private int countTableDataColumnsInPaintObjects(int[] columns,
			Set<String> objects) {
		int counter = 0;
		for (int column : columns) {
			if (objects.contains("" + column)) {
				counter += 1;
			}
		}
		return counter;
	}

	private int countTableDataRowsInPaintObjects(String[] objects) {
		try {
			return objects.length;
		} catch (Exception e) {
		}
		return 0;
	}

	private Map<String, Object> resetPaintObjects(
			Map<String, Object> paintObjects, int emptySearch,
			String newDataObject) {
		if (!getTableProperties().isOriginTableDataEmpty()) {
			// table is already visible on the client side
			paintObjects.put("noData", "noData");
		} else {
			// no data is send to the client yet
			paintObjects.put("emptyData", "emptyData");
		}

		getTableProperties().resetFilters(newDataObject);
		paintObjects.remove("newFilter");
		paintObjects.remove("removeFilter");

		if (getTableConfig().isEnabledQuickSearchClearOnNoNewData()
				&& "quickSearch".equals(newDataObject)) {
			getTableProperties().resetQuickSearch(newDataObject);
			paintObjects.put("emptyQuickSearch", "emptyQuickSearch");
		}

		if (getTableConfig().isEnabledSearchClearOnNoNewData()
				&& emptySearch >= 0) {
			paintObjects.put("emptySearch", emptySearch);
			getTableProperties().resetSearch(newDataObject);
		}

		return paintObjects;
	}

	private void initTable(boolean createArchive) {
		// set default locale
		if (getTableConfig().getLocale() == null) {
			getTableConfig().setLocale(getLocale());
		} else {
			setLocale(getTableConfig().getLocale());
		}

		// set needed translations
		getTableProperties().setTranslatedWordIsEmpty(
				getTranslation().get(getTableConfig().getLocale(),
						TranslatedWord.IS_EMPTY));
		getTableProperties().setTranslatedWordIsNotEmpty(
				getTranslation().get(getTableConfig().getLocale(),
						TranslatedWord.IS_NOT_EMPTY));

		if (!createArchive) {
			getTableConfig()
					.setAllColumns(getTableProperties().getAllColumns());

			// check frozen columns
			if (!getTableConfig().isEnabledFrozenTable()) {
				getTableProperties().resetFrozenColumns();
			}

			getTableProperties().checkVisibleColumns(
					getTableConfig().getTableWidth(),
					getTableConfig().getColumnsWidth(),
					getTableConfig().isEnabledAllColumns());

			// check for null headers in order objects
			Order[] orders = getTableProperties().getOrders();
			orders = OrderProcessor.process(orders, getTableProperties()
					.getCompleteColumnsHeader());
			getTableProperties().resetOrders();
			getTableProperties().setOrders(orders);

			Filter[] filters = getTableProperties().getCompleteFilter();
			Filter[] quickSearch = getTableProperties()
					.getQuickSearchFormatting(
							getTableConfig().getFormattingColumnsArray(),
							getTableConfig().isEnabledQuickSearchFromatting());
			getTableProperties().setAllRows(getAllRows(filters, quickSearch));
			getTableProperties().calculateTableWidth(
					getTableConfig().getColumnsWidth());

			prepareTableQuery(orders, filters, quickSearch, 0, getTableConfig()
					.getPageSize());

			int[] columns = getTableProperties().getFrozenAndVisibleColumns();
			int[] visibleHeaderColumns = getTableProperties()
					.getFrozenAndAllVisibleColumns();
			if (!getTableConfig().isEnabledAllColumns()
					&& !getTableConfig().header().isEnabledAllColumns()) {
				visibleHeaderColumns = columns;
			}
			paintObjects.put("visibleHeaderColumns", visibleHeaderColumns);
			paintObjects.put("columnsHeader", getTableProperties()
					.getColumnsHeader(visibleHeaderColumns));
			if (getTableConfig().header().isEnabledTooltip()) {
				paintObjects.put("columnsTooltip", getTableProperties()
						.getHeadersTooltip(visibleHeaderColumns));
			}

			paintObjects.putAll(getTableConfig().getPaintObjects(columns,
					visibleHeaderColumns));
			paintObjects.putAll(getTableProperties().getPaintObjects());

			Map<String, String[]> newTableData = getTableColumnsData(
					getTableProperties().getFirstRow(), getTableConfig()
							.getPageSize(), columns, getTableProperties()
							.getOrders(), filters, quickSearch, true);
			if (newTableData != null) {
				paintObjects.putAll(newTableData);
				frozenAndVisibleTableColumnsLength = columns.length;
			}

			if (countTableDataColumnsInPaintObjects(columns,
					paintObjects.keySet()) > 0) {
				paintObjects
						.put("allRowsVisible",
								countTableDataRowsInPaintObjects((String[]) paintObjects
										.get("" + columns[0])) == getTableProperties()
										.getAllRows());
			} else {
				paintObjects.put("emptyData", "emptyData");
			}
		} else {
			isTableInitialized = true;
			archivedPaintObjects = new HashMap<String, Object>(paintObjects);
		}
	}

	public void refreshLayout(int pageRefresh) {
		paintObjects.put("pageRefresh", pageRefresh > 0 ? pageRefresh
				: TableConfig.DEFAULT_PAGE_REFRESH);
	}

	public void refreshVisibleData(String cause, boolean showDialog) {
		paintObjectsIsProcessedNormal = true;
		paintObjects.put("refreshVisibleData", Boolean.valueOf(showDialog));
		paintObjects.put("refreshVisibleDataCause", cause);
		requestRepaint();
	}

	public boolean prepareExportedData(String name) {
		Order[] orders = getTableProperties().getOrders();
		Filter[] filters = getTableProperties().getCompleteFilter();
		Filter[] quickSearch = getTableProperties().getQuickSearchFormatting(
				getTableConfig().getFormattingColumnsArray(),
				getTableConfig().isEnabledQuickSearchFromatting());
		return prepareExportedData(name, orders, filters, quickSearch);
	}

	public long getExportedAllRows(String name) {
		Filter[] filters = getTableProperties().getCompleteFilter();
		Filter[] quickSearch = getTableProperties().getQuickSearchFormatting(
				getTableConfig().getFormattingColumnsArray(),
				getTableConfig().isEnabledQuickSearchFromatting());

		prepareTableQuery(null, filters, quickSearch, 0, 0);
		getTableExporter().setAllRows(name,
				getExportedAllRows(name, filters, quickSearch));

		return getTableExporter().getAllRows(name);
	}

	public Object[] getExportedData(String name, int startRow, int rows) {
		Order[] orders = getTableProperties().getOrders();
		Filter[] filters = getTableProperties().getCompleteFilter();
		Filter[] quickSearch = getTableProperties().getQuickSearchFormatting(
				getTableConfig().getFormattingColumnsArray(),
				getTableConfig().isEnabledQuickSearchFromatting());

		prepareTableQuery(orders, filters, quickSearch, startRow, rows);

		boolean enabledAutoPrepareDataWithoutHiddenColumns = getTableConfig()
				.isEnabledAutoPrepareDataWithoutHiddenColumns();
		boolean enabledAutoPrepareDataWithoutRemovedColumns = getTableConfig()
				.isEnabledAutoPrepareDataWithoutRemovedColumns();
		int[] removedColumns = getTableProperties().getHiddenAndRemovedColumns(
				enabledAutoPrepareDataWithoutHiddenColumns,
				enabledAutoPrepareDataWithoutRemovedColumns);

		if (name.equals(getTableExporter().getBasename())
				&& removedColumns.length > 0) {
			int newDataCounter = 0;
			int allColumns = getTableProperties().getAllColumns();
			Object[] data = getExportedData(name, startRow, rows, orders,
					filters, quickSearch);
			Object[] newData = new Object[rows
					* (allColumns - removedColumns.length)];
			for (int row = 0; row < rows; row++) {
				for (int column = 0; column < allColumns; column++) {
					if (!ArrayUtils.contains(removedColumns, column)) {
						newData[newDataCounter++] = data[row * allColumns
								+ column];
					}
				}
			}
			return newData;
		}

		return getExportedData(name, startRow, rows, orders, filters,
				quickSearch);
	}

	@Override
	public void paintContent(PaintTarget target) throws PaintException {
		super.paintContent(target);

		// set default locale
		if (getTableConfig().getLocale() == null) {
			getTableConfig().setLocale(getLocale());
		} else {
			setLocale(getTableConfig().getLocale());
		}

		if (!isTableInitialized || !paintObjectsIsProcessedNormal) {
			if (!isTableInitialized) {
				initTable(false);
			} else if (!paintObjectsIsProcessedNormal) {
				// page reload
				paintObjects = new HashMap<String, Object>(archivedPaintObjects);
			}
			target.addAttribute("tableConfig", getTableConfig()
					.getPaintObjects());
			target.addAttribute("tableExport", getTableExporter()
					.getPaintObjects());
			target.addAttribute("translation", getTranslation()
					.getPaintObjects(getTableConfig().getLocale()));
		} else {
			paintObjectsIsProcessedNormal = false;
			if (!paintObjects.containsKey("refreshVisibleData")) {
				paintObjects.putAll(getTableProperties().getPaintObjects());
			}
		}

		for (Map.Entry<String, Object> entry : paintObjects.entrySet()) {
			String name = entry.getKey();
			Object value = entry.getValue();
			if (value instanceof Boolean) {
				target.addAttribute(name, (Boolean) value);
			} else if (value instanceof Double) {
				target.addAttribute(name, (Double) value);
			} else if (value instanceof Float) {
				target.addAttribute(name, (Float) value);
			} else if (value instanceof Integer) {
				target.addAttribute(name, (Integer) value);
			} else if (value instanceof Long) {
				target.addAttribute(name, (Long) value);
			} else if (value instanceof String) {
				target.addAttribute(name, (String) value);
			} else if (value instanceof String[] || value instanceof Integer[]) {
				target.addAttribute(name, (Object[]) value);
			} else if (value instanceof int[]) {
				target.addAttribute(name,
						(Object[]) ArrayUtils.toObject((int[]) value));
			}
		}

		// create archive paint objects
		if (!isTableInitialized) {
			initTable(true);
		}
	}

	@SuppressWarnings("unchecked")
	@Override
	public void changeVariables(Object source, Map<String, Object> variables) {
		super.changeVariables(source, variables);
		Map<String, Object> request = (Map<String, Object>) variables
				.get("request");

		if (request == null || request.isEmpty()
				|| request.containsKey("pageReload")) {
			paintObjectsIsProcessedNormal = false;
			requestRepaint();
		} else if (request.containsKey("transmissionError")
				|| !request.containsKey("top") || !request.containsKey("left")
				|| !request.containsKey("width")
				|| !request.containsKey("pageSize")
				|| !request.containsKey("bodyRowHeight")) {
			getApplication()
					.getMainWindow()
					.showNotification(
							getTranslation().get(getTableConfig().getLocale(),
									TranslatedWord.ERROR),
							getTranslation()
									.get(getTableConfig().getLocale(),
											TranslatedWord.WHEN_TRANSMITTING_DATA_AN_ERROR_HAS_OCCURRED),
							Notification.TYPE_ERROR_MESSAGE);
		} else if (request.containsKey("noData")
				&& getTableConfig().isEnabledNoNewDataMessage()) {
			getApplication().getMainWindow().showNotification(
					getTranslation().get(getTableConfig().getLocale(),
							TranslatedWord.NO_NEW_DATA),
					getTranslation().get(getTableConfig().getLocale(),
							TranslatedWord.SIZE_OF_THE_REQUESTED_DATA_IS_ZERO),
					Notification.TYPE_ERROR_MESSAGE);
		} else if (request.containsKey("dataError")) {
			getApplication()
					.getMainWindow()
					.showNotification(
							getTranslation().get(getTableConfig().getLocale(),
									TranslatedWord.DATA_ERROR),
							getTranslation()
									.get(getTableConfig().getLocale(),
											TranslatedWord.SOMETHING_IS_WRONG_IN_DATA_PREPARATION_PLEASE_CHECK_YOUR_DATA_SETTINGS),
							Notification.TYPE_ERROR_MESSAGE);
		} else if (request.containsKey("export")
				|| request.containsKey("columnsAction")) {
			if (windowsHelper == null) {
				windowsHelper = new WindowsHelper(this);
			} else if (windowsHelper.getWindow() != null
					&& windowsHelper.getWindow().getParent() != null) {
				getApplication().getMainWindow().removeWindow(
						windowsHelper.getWindow());
			}
			windowsHelper.process(request.get(
					request.containsKey("export") ? "export" : "columnsAction")
					.toString());
		} else { // standard message elements
			boolean fetchNewData = false;
			int emptySearch = -1;
			String newDataObject = new String();
			Map<String, Object> paintObjects = new HashMap<String, Object>();

			if (getTableConfig().isEnabledFilteredFrozenTable()
					|| getTableConfig().isEnabledFilteredTable()) {
				if (request.containsKey("newFilter")) {
					String[] newFilter = request.get("newFilter").toString()
							.split("_::_");
					int column = Integer.parseInt(newFilter[0]);
					int row = Integer.parseInt(newFilter[1]);
					Filter filter = FilterProcessor.process(newFilter,
							getTableProperties().getFilter(column),
							getTableProperties()
									.getOriginTableData(column, row),
							getTableProperties().getColumnHeader(column));
					getTableProperties().setFilter(filter);
					fetchNewData = Boolean.TRUE;
					newDataObject = "newFilter";
				} else if (request.containsKey("removeFilter")) {
					int column = Integer.parseInt(request.get("removeFilter")
							.toString());
					getTableProperties().removeFilter(column);
					getTableProperties().removeSearch(column);
					fetchNewData = Boolean.TRUE;
				} else if (request.containsKey("quickSearch")
						&& getTableConfig().isEnabledQuickSearch()) {
					getTableProperties().setQuickSearch(
							request.get("quickSearch").toString());
					fetchNewData = Boolean.TRUE;
					newDataObject = "quickSearch";
				} else if (request.containsKey("search")
						&& getTableConfig().isEnabledSearch()) {
					String searchString = request.get("search").toString();
					if (searchString != null && !searchString.isEmpty()
							&& !"none".equals(searchString)) {
						String[] searchValues = searchString.split("_::_");
						int column = Integer.parseInt(searchValues[0]);
						Search search = SearchProcessor.process(searchValues,
								getTableProperties().getColumnHeader(column),
								getTableConfig().getFormattingColumn(column));
						if (search != null) {
							if (search.getOriginalValue() != null) {
								getTableProperties().setSearches(search);
								fetchNewData = Boolean.TRUE;
							} else if (search.getOriginalValue() == null) {
								// error parsing format for search value
								String error = "searchError";
								paintObjects.put("noData", error);
								paintObjects.put(error, search.getColumn());
								fetchNewData = Boolean.FALSE;
							}
						} else {
							getTableProperties().removeSearch(column);
							fetchNewData = Boolean.TRUE;
						}
						emptySearch = column;
					} else if ("none".equals(searchString)
							&& !getTableProperties().isNoSearches()) {
						getTableProperties().removeSearch();
						fetchNewData = Boolean.TRUE;
					}
					newDataObject = "search";
				}
			}

			if (request.containsKey("order")) {
				if (getTableConfig().isEnabledOrderedFrozenTable()
						|| getTableConfig().isEnabledOrderedTable()) {
					Object orderObject = request.get("order");
					if ("none".equals(orderObject)) {
						getTableProperties().resetOrders();
					} else {
						getTableProperties().resetOrders();
						getTableProperties().setOrders(
								OrderProcessor.process(orderObject,
										getTableProperties()
												.getCompleteColumnsHeader()));
					}
					fetchNewData = Boolean.TRUE;
				} else {
					getTableProperties().resetOrders();
					fetchNewData = Boolean.FALSE;
				}
			}

			if (request.containsKey("freeze")) {
				if (getTableConfig().isEnabledFrozenTable()) {
					int column = Integer.parseInt(request.get("freeze")
							.toString());
					if (getTableProperties().isFrozenColumn(column)) {
						getTableProperties().removeFrozenColumn(column);
					} else {
						getTableProperties().setFrozenColumn(column);
					}
					getTableProperties().calculateTableWidth(
							getTableConfig().getColumnsWidth());
					fetchNewData = Boolean.TRUE;
				} else {
					getTableProperties().resetFrozenColumns();
					fetchNewData = Boolean.FALSE;
				}
			}

			if (request.containsKey("hideColumn")) {
				if ("refreshVisibleDataCause".equals(request.get("hideColumn")
						.toString())) {
					fetchNewData = getTableConfig()
							.isEnabledNewDataWhenHideUnhideColumn();
				} else {
					int column = Integer.parseInt(request.get("hideColumn")
							.toString());
					if (getTableConfig().canColumnHide(column)) {
						getTableProperties().setHiddenColumn(column);
						fetchNewData = getTableConfig()
								.isEnabledNewDataWhenHideUnhideColumn();
					}
				}
				if (fetchNewData) {
					getTableProperties().calculateTableWidth(
							getTableConfig().getColumnsWidth());
				}
			}

			if (getTableConfig().isEnabledFilteredFrozenTable()
					|| getTableConfig().isEnabledFilteredTable()
					|| request.containsKey("order")) {
				paintObjects.put("newFilter", getTableProperties()
						.getNewFilter());
				paintObjects.put("filterHtml", getTableProperties()
						.getFilterHtml());
				paintObjects.put("search", getTableProperties()
						.getSearchString());
			}

			if (request.containsKey("newData")
					|| request.containsKey("newFilter")
					|| request.containsKey("removeFilter")
					|| request.containsKey("quickSearch")
					|| request.containsKey("search")
					|| request.containsKey("order")
					|| request.containsKey("freeze")
					|| request.containsKey("hideColumn")) {
				if (request.containsKey("newData")) {
					fetchNewData = Boolean.parseBoolean(request.get("newData")
							.toString());
				}

				Filter[] filters = getTableProperties().getCompleteFilter();
				Filter[] quickSearch = getTableProperties()
						.getQuickSearchFormatting(
								getTableConfig().getFormattingColumnsArray(),
								getTableConfig()
										.isEnabledQuickSearchFromatting());
				if (!paintObjects.containsKey("noData")
						&& (request.containsKey("newFilter")
								|| request.containsKey("removeFilter")
								|| request.containsKey("quickSearch") || request
									.containsKey("search"))) {
					prepareTableQuery(null, filters, quickSearch, 0, 0);

					long allRows = getAllRows(filters, quickSearch);
					if (allRows > 0) {
						fetchNewData = true;
						getTableProperties().setAllRows(allRows);
					} else {
						paintObjects = resetPaintObjects(paintObjects,
								emptySearch, newDataObject);
					}
				}

				if (!paintObjects.containsKey("noData")
						&& !paintObjects.containsKey("emptyData")) {
					getTableProperties().checkVisibleColumns(
							Integer.parseInt(request.get("width").toString()),
							Integer.parseInt(request.get("left").toString()),
							Integer.parseInt(request.get("top").toString()),
							Integer.parseInt(request.get("bodyRowHeight")
									.toString()),
							getTableConfig().getColumnsWidth(),
							getTableConfig().isEnabledAllColumns());
					int[] columns = getTableProperties()
							.getFrozenAndVisibleColumns();

					// need to send this data to the client only if:
					// - number of visible columns are not equal columns header
					// - table is scrolling horizontally
					// - columns is hidden and fetch new data is enabled
					// - columns is freezing
					// - columns is ordered
					if (refreshTableHeader(columns.length, fetchNewData,
							request.containsKey("hideColumn"),
							request.containsKey("freeze"),
							request.containsKey("order"))) {
						int[] visibleHeaderColumns = getTableProperties()
								.getFrozenAndAllVisibleColumns();
						if (!getTableConfig().isEnabledAllColumns()
								&& !getTableConfig().header()
										.isEnabledAllColumns()) {
							visibleHeaderColumns = columns;
						}
						paintObjects.put("visibleHeaderColumns",
								visibleHeaderColumns);
						paintObjects.put("columnsHeader", getTableProperties()
								.getColumnsHeader(visibleHeaderColumns));
						paintObjects.putAll(getTableConfig().getPaintObjects(
								columns, visibleHeaderColumns));
						paintObjects.putAll(getTableProperties()
								.getPaintObjects());
						if (getTableConfig().header().isEnabledTooltip()) {
							paintObjects.put(
									"headersTooltip",
									getTableProperties().getHeadersTooltip(
											visibleHeaderColumns));
						}
					}
					// new table data
					int firstRow = getTableProperties().getFirstRow();
					int rows = Integer.parseInt(request.get("pageSize")
							.toString());
					rows = (int) (firstRow + rows < getTableProperties()
							.getAllRows() ? rows : getTableProperties()
							.getAllRows() - firstRow);
					Map<String, String[]> newTableData = getTableColumnsData(
							firstRow, rows, columns, getTableProperties()
									.getOrders(), filters, quickSearch,
							fetchNewData);
					if (newTableData != null) {
						paintObjects.putAll(newTableData);
						frozenAndVisibleTableColumnsLength = columns.length;
					}

					if (countTableDataColumnsInPaintObjects(columns,
							paintObjects.keySet()) > 0) {
						paintObjects
								.put("allRowsVisible",
										countTableDataRowsInPaintObjects((String[]) paintObjects
												.get("" + columns[0])) == getTableProperties()
												.getAllRows());
					} else {
						paintObjects = resetPaintObjects(paintObjects,
								emptySearch, newDataObject);
					}
				}
				// send new data to the client
				paintObjectsIsProcessedNormal = true;
				this.paintObjects = new HashMap<String, Object>(paintObjects);
				requestRepaint();
			}
		}
	}

	private Map<String, String[]> getTableColumnsData(int startRow, int rows,
			int[] columns, Order[] orders, Filter[] filters,
			Filter[] quickSearch, boolean fetchNewData) {
		if (fetchNewData) {
			if (getTableConfig().isEnabledAllRows()) {
				startRow = 0;
				rows = (int) getTableProperties().getAllRows();
			}

			prepareTableQuery(orders, filters, quickSearch, startRow, rows);
			Object[] data = getRowsData(startRow, rows, orders, filters,
					quickSearch);

			if (data != null && data.length > 0) {
				transformTableRows2ColumnsData(data);
			} else {
				return null;
			}
		}
		return getVisibleTableColumnsData(columns);
	}

	private void transformTableRows2ColumnsData(Object[] rowsData) {
		getTableProperties().resetOriginTableData();
		int allRows = getTableProperties().getAllColumns() == 0 ? 0
				: rowsData.length / getTableProperties().getAllColumns();
		for (int column = 0; column < getTableProperties().getAllColumns(); column++) {
			List<Object> columnData = new ArrayList<Object>();
			for (int row = 0; row < allRows; row++) {
				columnData.add(rowsData[getTableProperties().getAllColumns()
						* row + column]);
			}
			getTableProperties().setOriginTableData(columnData);
		}
	}

	private Map<String, String[]> getVisibleTableColumnsData(int[] columns) {
		Map<String, String[]> tempColumnsData = new HashMap<String, String[]>();
		for (int column : columns) {
			if (column < getTableProperties().getAllColumns()) {
				tempColumnsData.put("" + column, FormattingProcessor.process(
						getTableConfig().getFormattingColumn(column),
						getTableProperties().getOriginTableData(column)));
			}
		}
		return tempColumnsData;
	}

	public String setImage(String url, int width, int height) {
		return ("image_:_" + url + "_:_" + width + "_:_" + height);
	}

	public void setColumn(int column, String columnHeader) {
		column = column >= 0 ? column : 0;
		getTableProperties().setColumnHeader(column, columnHeader);
	}

	public void setColumn(int column, String columnHeader, int columnWidth) {
		column = column >= 0 ? column : 0;
		getTableProperties().setColumnHeader(column, columnHeader);
		getTableConfig().setColumnWidth(column, columnWidth);
	}

	public void setColumn(int column, String columnHeader, int columnWidth,
			ColumnAlign bodyColumnAlign) {
		column = column >= 0 ? column : 0;
		getTableProperties().setColumnHeader(column, columnHeader);
		getTableConfig().setColumnWidth(column, columnWidth);
		getTableConfig().body().setColumnAlignment(column, bodyColumnAlign);
	}

	public void setColumn(int column, String columnHeader, int columnWidth,
			ColumnAlign headerColumnAlign, ColumnAlign bodyColumnAlign) {
		column = column >= 0 ? column : 0;
		getTableProperties().setColumnHeader(column, columnHeader);
		getTableConfig().setColumnWidth(column, columnWidth);
		getTableConfig().header().setColumnAlignment(column, headerColumnAlign);
		getTableConfig().body().setColumnAlignment(column, bodyColumnAlign);
	}

	public void setColumn(int column, String columnHeader,
			ColumnAlign bodyColumnAlign) {
		column = column >= 0 ? column : 0;
		getTableProperties().setColumnHeader(column, columnHeader);
		getTableConfig().body().setColumnAlignment(column, bodyColumnAlign);
	}

	public void setColumn(int column, String columnHeader,
			ColumnAlign headerColumnAlign, ColumnAlign bodyColumnAlign) {
		column = column >= 0 ? column : 0;
		getTableProperties().setColumnHeader(column, columnHeader);
		getTableConfig().header().setColumnAlignment(column, headerColumnAlign);
		getTableConfig().body().setColumnAlignment(column, bodyColumnAlign);
	}

	public void setColumn(int column, String columnHeader, Format format) {
		column = column >= 0 ? column : 0;
		getTableProperties().setColumnHeader(column, columnHeader);
		getTableConfig().setFormattingColumn(column, format);
	}

	public void setColumn(int column, String columnHeader, int columnWidth,
			Format format) {
		column = column >= 0 ? column : 0;
		getTableProperties().setColumnHeader(column, columnHeader);
		getTableConfig().setColumnWidth(column, columnWidth);
		getTableConfig().setFormattingColumn(column, format);
	}

	public void setColumn(int column, String columnHeader, int columnWidth,
			ColumnAlign bodyColumnAlign, Format format) {
		column = column >= 0 ? column : 0;
		getTableProperties().setColumnHeader(column, columnHeader);
		getTableConfig().setColumnWidth(column, columnWidth);
		getTableConfig().body().setColumnAlignment(column, bodyColumnAlign);
		getTableConfig().setFormattingColumn(column, format);
	}

	public void setColumn(int column, String columnHeader, int columnWidth,
			ColumnAlign headerColumnAlign, ColumnAlign bodyColumnAlign,
			Format format) {
		column = column >= 0 ? column : 0;
		getTableProperties().setColumnHeader(column, columnHeader);
		getTableConfig().setColumnWidth(column, columnWidth);
		getTableConfig().header().setColumnAlignment(column, headerColumnAlign);
		getTableConfig().body().setColumnAlignment(column, bodyColumnAlign);
		getTableConfig().setFormattingColumn(column, format);
	}

	public void setColumn(int column, String columnHeader,
			ColumnAlign bodyColumnAlign, Format format) {
		column = column >= 0 ? column : 0;
		getTableProperties().setColumnHeader(column, columnHeader);
		getTableConfig().body().setColumnAlignment(column, bodyColumnAlign);
		getTableConfig().setFormattingColumn(column, format);
	}

	public void setColumn(int column, String columnHeader,
			ColumnAlign headerColumnAlign, ColumnAlign bodyColumnAlign,
			Format format) {
		column = column >= 0 ? column : 0;
		getTableProperties().setColumnHeader(column, columnHeader);
		getTableConfig().header().setColumnAlignment(column, headerColumnAlign);
		getTableConfig().body().setColumnAlignment(column, bodyColumnAlign);
		getTableConfig().setFormattingColumn(column, format);
	}

	public void setColumn(int column, String columnHeader, String headerTooltip) {
		column = column >= 0 ? column : 0;
		getTableProperties().setColumnHeader(column, columnHeader);
		getTableProperties().setHeaderTooltip(column, headerTooltip);
	}

	public void setColumn(int column, String columnHeader,
			String headerTooltip, int columnWidth) {
		column = column >= 0 ? column : 0;
		getTableProperties().setColumnHeader(column, columnHeader);
		getTableProperties().setHeaderTooltip(column, headerTooltip);
		getTableConfig().setColumnWidth(column, columnWidth);
	}

	public void setColumn(int column, String columnHeader,
			String headerTooltip, int columnWidth, ColumnAlign bodyColumnAlign) {
		column = column >= 0 ? column : 0;
		getTableProperties().setColumnHeader(column, columnHeader);
		getTableProperties().setHeaderTooltip(column, headerTooltip);
		getTableConfig().setColumnWidth(column, columnWidth);
		getTableConfig().body().setColumnAlignment(column, bodyColumnAlign);
	}

	public void setColumn(int column, String columnHeader,
			String headerTooltip, int columnWidth,
			ColumnAlign headerColumnAlign, ColumnAlign bodyColumnAlign) {
		column = column >= 0 ? column : 0;
		getTableProperties().setColumnHeader(column, columnHeader);
		getTableProperties().setHeaderTooltip(column, headerTooltip);
		getTableConfig().setColumnWidth(column, columnWidth);
		getTableConfig().header().setColumnAlignment(column, headerColumnAlign);
		getTableConfig().body().setColumnAlignment(column, bodyColumnAlign);
	}

	public void setColumn(int column, String columnHeader,
			String headerTooltip, ColumnAlign bodyColumnAlign) {
		column = column >= 0 ? column : 0;
		getTableProperties().setColumnHeader(column, columnHeader);
		getTableProperties().setHeaderTooltip(column, headerTooltip);
		getTableConfig().body().setColumnAlignment(column, bodyColumnAlign);
	}

	public void setColumn(int column, String columnHeader,
			String headerTooltip, ColumnAlign headerColumnAlign,
			ColumnAlign bodyColumnAlign) {
		column = column >= 0 ? column : 0;
		getTableProperties().setColumnHeader(column, columnHeader);
		getTableProperties().setHeaderTooltip(column, headerTooltip);
		getTableConfig().header().setColumnAlignment(column, headerColumnAlign);
		getTableConfig().body().setColumnAlignment(column, bodyColumnAlign);
	}

	public void setColumn(int column, String columnHeader,
			String headerTooltip, Format format) {
		column = column >= 0 ? column : 0;
		getTableProperties().setColumnHeader(column, columnHeader);
		getTableProperties().setHeaderTooltip(column, headerTooltip);
		getTableConfig().setFormattingColumn(column, format);
	}

	public void setColumn(int column, String columnHeader,
			String headerTooltip, int columnWidth, Format format) {
		column = column >= 0 ? column : 0;
		getTableProperties().setColumnHeader(column, columnHeader);
		getTableProperties().setHeaderTooltip(column, headerTooltip);
		getTableConfig().setColumnWidth(column, columnWidth);
		getTableConfig().setFormattingColumn(column, format);
	}

	public void setColumn(int column, String columnHeader,
			String headerTooltip, int columnWidth, ColumnAlign bodyColumnAlign,
			Format format) {
		column = column >= 0 ? column : 0;
		getTableProperties().setColumnHeader(column, columnHeader);
		getTableProperties().setHeaderTooltip(column, headerTooltip);
		getTableConfig().setColumnWidth(column, columnWidth);
		getTableConfig().body().setColumnAlignment(column, bodyColumnAlign);
		getTableConfig().setFormattingColumn(column, format);
	}

	public void setColumn(int column, String columnHeader,
			String headerTooltip, int columnWidth,
			ColumnAlign headerColumnAlign, ColumnAlign bodyColumnAlign,
			Format format) {
		column = column >= 0 ? column : 0;
		getTableProperties().setColumnHeader(column, columnHeader);
		getTableProperties().setHeaderTooltip(column, headerTooltip);
		getTableConfig().setColumnWidth(column, columnWidth);
		getTableConfig().header().setColumnAlignment(column, headerColumnAlign);
		getTableConfig().body().setColumnAlignment(column, bodyColumnAlign);
		getTableConfig().setFormattingColumn(column, format);
	}

	public void setColumn(int column, String columnHeader,
			String headerTooltip, ColumnAlign bodyColumnAlign, Format format) {
		column = column >= 0 ? column : 0;
		getTableProperties().setColumnHeader(column, columnHeader);
		getTableProperties().setHeaderTooltip(column, headerTooltip);
		getTableConfig().body().setColumnAlignment(column, bodyColumnAlign);
		getTableConfig().setFormattingColumn(column, format);
	}

	public void setColumn(int column, String columnHeader,
			String headerTooltip, ColumnAlign headerColumnAlign,
			ColumnAlign bodyColumnAlign, Format format) {
		column = column >= 0 ? column : 0;
		getTableProperties().setColumnHeader(column, columnHeader);
		getTableProperties().setHeaderTooltip(column, headerTooltip);
		getTableConfig().header().setColumnAlignment(column, headerColumnAlign);
		getTableConfig().body().setColumnAlignment(column, bodyColumnAlign);
		getTableConfig().setFormattingColumn(column, format);
	}

	@Override
	public String toString() {
		return getTableExporter().getBasename();
	}

}
