/**
 * Copyright (C) 2012  Dušan Vejnovič  <vaadin@dussan.org>
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 * http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */
package org.dussan.vaadin.dtable;

import java.text.DecimalFormat;
import java.text.SimpleDateFormat;
import java.util.ArrayList;
import java.util.Date;
import java.util.List;
import java.util.Locale;

import org.dussan.vaadin.dtable.config.TableToolsPosition;
import org.dussan.vaadin.dtable.config.TooltipPosition;
import org.dussan.vaadin.dtable.config.tablesection.alignments.ColumnAlign;
import org.dussan.vaadin.dtable.config.tablesection.alignments.RowAlign;
import org.dussan.vaadin.dtable.properties.filter.Filter;
import org.dussan.vaadin.dtable.properties.order.Order;
import org.dussan.vaadin.dtable.properties.order.OrderType;

public class DemoPagableTable extends DTable {

	private static final long serialVersionUID = -7006130816222005690L;
	private int allRows = 6;
	private int allColumns = 20;

	public DemoPagableTable() {
		super("demoPagableTable");
		getTableConfig().setLocale(new Locale("sl", "SI"));
		getTableConfig().enablePageableTable(true);
		getTableConfig().enableOrderedTable(true);
		getTableConfig().enableFilteredTable(true);
		getTableConfig().enableFilterInfo(true);
		getTableConfig().enableHighlightActiveRow(true);
		getTableConfig().enableFixedTableHeight(true);
		getTableConfig().enableOddAndEvenColoringRows(true);
		getTableConfig().setPageSize(10);
		getTableConfig().setPagerPages(15);
		getTableConfig().setColumnDefaultWidth(70);

		getTableConfig().setTableToolsPosition(TableToolsPosition.BOTTOM_LEFT);
		getTableConfig().enableSearch(true);
		getTableConfig().enableSearchHide(true);
		getTableConfig().enableSearchVisibleAtStartup(true);
		getTableConfig().enableQuickSearch(true);
		getTableConfig().enableQuickSearchHide(true);
		getTableConfig().enableQuickSearchVisibleAtStartup(true);

		getTableConfig().enableFreezeAnyColumns(true);
		getTableConfig().enableUnfreezeAnyColumns(true);
		getTableConfig().enableColumnUnfreeze(true, 1);

		getTableConfig().enableLoadingMessage(true);
		getTableConfig().setLoadingMessageOpacity(0.5f);
		getTableConfig().setLoadingMessageDelay(100, 300);

		getTableConfig().enableFrozenTable(true);
		getTableConfig().enableFrozenTableVisible(true);
		getTableConfig().enableOrderedFrozenTable(false);
		getTableConfig().enableFilteredFrozenTable(false);
		getTableConfig().enableHideFrozenTable(true);
		getTableConfig().enableResizeFrozenTable(true);
		getTableConfig().enableEmptyFrozenTable(true);
		getTableConfig().setTableFrozenMinimalWidth(200);

		getTableConfig().header().enableVerticalLines(true);
		getTableConfig().header().enableExpandedCell(true, true);
		getTableConfig().header().enableExpandedCellForActiveColumn(true);
		getTableConfig().header().enableExpandedFrozenCell(true, true);
		getTableConfig().header().enableExpandedFrozenCellForActiveColumn(true);
		getTableConfig().header().setRowHeight(24);
		getTableConfig().header().enableWordWrap(false);
		getTableConfig().header().setColumnsAlignment(ColumnAlign.LEFT,
				ColumnAlign.LEFT, ColumnAlign.LEFT, ColumnAlign.LEFT,
				ColumnAlign.LEFT, ColumnAlign.LEFT, ColumnAlign.LEFT);
		getTableConfig().header().enableTooltip(true);
		getTableConfig().header().enableWordWrapInTooltip(true);
		getTableConfig().header().setTooltipMaximumWidth(130);
		getTableConfig().header().setTooltipPosition(TooltipPosition.BOTTOM);

		getTableConfig().body().enableVerticalLines(true);
		getTableConfig().body().setRowHeight(24);
		getTableConfig().body().setRowAlignment(RowAlign.BOTTOM);
		getTableConfig().body().enableWordWrap(true);
		getTableConfig().body().enableExpandedFrozenCell(true);
		getTableConfig().body().enableExpandedCell(true);
		getTableConfig().body().setDefaultColumnAlignment(ColumnAlign.CENTER);

		getTableProperties().setFrozenColumns(1, 2);
		getTableProperties().setOrders(OrderType.ASCEDING, 1, 6);
		getTableProperties().setOrders(OrderType.DESCEDING, 4, 5);

		for (int column = 1; column <= allColumns; column++) {
			if (column == 1 || column == 5) {
				setColumn(column - 1, "HEADER " + column, 30);
			} else if (column == 3 || column == 10) {
				setColumn(column - 1, "HEADER " + column, "Datum",
						new SimpleDateFormat("dd.MM.yyyy HH:mm:ss"));
			} else {
				setColumn(column - 1, "HEADER " + column, "Denar",
						ColumnAlign.RIGHT, ColumnAlign.RIGHT,
						new DecimalFormat("#,##0.00"));
			}
		}
	}

	@Override
	public Object[] getRowsData(int startRow, int rows, Order[] sortings,
			Filter[] filterings, Filter[] quickSearch) {
		rows = rows == 0 ? allRows : rows;

		List<Object> data = new ArrayList<Object>();
		for (int row = startRow; row < (startRow + rows)
				&& row < getAllRows(filterings, quickSearch); row++) {
			for (int column = 0; column < allColumns; column++) {
				if (column == 0) {
					String url = getApplication().getURL()
							+ "VAADIN/widgetsets/org.dussan.vaadin.dtable.DtableWidgetset/dtable-css/images/sort_desc.png";
					data.add(setImage(url, 16, 16));
				} else if (column == 3 || column == 10) {
					data.add(new Date());
				} else if (column == 5) {
					data.add("Testing, testing, testing, testing, testing, ...");
				} else {
					data.add((float) row * 100 + (float) column / 100);
				}
			}
		}
		return (filterings != null && filterings.length > 1 ? null : data
				.toArray());
	}

	@Override
	public long getAllRows(Filter[] filterings, Filter[] quickSearch) {
		if (filterings != null && filterings.length > 2) {
			return 0;
		}
		return (filterings == null || filterings.length == 0 ? allRows
				: allRows / (filterings.length + 1));
	}

	@Override
	public boolean prepareExportedData(String name, Order[] sortings,
			Filter[] filterings, Filter[] quickSearch) {
		return true;
	}

	@Override
	public long getExportedAllRows(String name, Filter[] filterings,
			Filter[] quickSearch) {
		return allRows;
	}

	@Override
	public Object[] getExportedData(String name, int startRow, int rows,
			Order[] sortings, Filter[] filterings, Filter[] quickSearch) {
		List<Object> data = new ArrayList<Object>();
		for (int row = startRow; row < (startRow + rows); row++) {
			for (int column = 0; column < allColumns; column++) {
				if (column == 0) {
					String url = getApplication().getURL()
							+ "VAADIN/widgetsets/org.dussan.vaadin.dtable.DtableWidgetset/dtable-css/images/sort_desc.png";
					data.add(setImage(url, 16, 16));
				} else if (column == 3 || column == 10) {
					data.add(new Date());
				} else if (column == 5) {
					data.add("Testing, testing, testing, testing, testing, ...");
				} else {
					data.add((float) row * 100 + (float) column / 100);
				}
			}
		}
		return data.toArray();
	}

	@Override
	public void exportProcessCompleted() {
	}

}
