/**
 * Copyright (C) 2012  Dušan Vejnovič  <vaadin@dussan.org>
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 * http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */
package org.dussan.vaadin.dtable.client.components.table;

import static com.google.gwt.query.client.GQuery.$;

import java.util.ArrayList;
import java.util.Collections;
import java.util.HashMap;
import java.util.List;
import java.util.Map;

import org.dussan.vaadin.dtable.client.components.base.TableComponent;
import org.dussan.vaadin.dtable.client.components.events.TableEvent;
import org.dussan.vaadin.dtable.client.ui.VDTable;

import com.google.gwt.dom.client.Style.Position;
import com.google.gwt.query.client.Function;
import com.google.gwt.query.client.GQuery;
import com.google.gwt.query.client.css.CSS;
import com.google.gwt.query.client.css.Length;
import com.google.gwt.query.client.css.RGBColor;
import com.google.gwt.query.client.css.TakesCssValue.CssSetter;
import com.google.gwt.query.client.css.TextAlignProperty.TextAlign;
import com.google.gwt.user.client.Element;
import com.google.gwt.user.client.Event;
import com.vaadin.terminal.gwt.client.BrowserInfo;
import com.vaadin.terminal.gwt.client.UIDL;

public class TableBody extends TableComponent {

	private String frozenHeaderId = "table#dt-frozen-header-table";
	private String headerId = "table#dt-header-table";
	private String frozenBodyId = "table#dt-frozen-body-table";
	private String bodyId = "table#dt-body-table";

	private Map<Integer, String[]> tableData = new HashMap<Integer, String[]>();
	private Map<Integer, Integer> columnAlignments = new HashMap<Integer, Integer>();

	private GQuery frozenRowHighlightEventHandler = null;
	private GQuery frozenCellHoverEventHandler = null;
	private GQuery frozenCellContextMenuEventHandler = null;
	private GQuery rowHighlightEventHandler = null;
	private GQuery cellMouseOverEventHandler = null;
	private GQuery cellContextMenuEventHandler = null;

	public TableBody(VDTable dTable) {
		super(dTable);
	}

	private void removeExpandedCells() {
		if (getDTable().config().isEnabledFrozenTable()) {
			$(frozenBodyId).find("div#clone").parent()
					.children("div.normal", "div.nowrap").css(CSS.DISPLAY, "")
					.parent().children("div#clone").remove();
		}
		$(bodyId).find("div#clone").parent()
				.children("div.normal", "div.nowrap").css(CSS.DISPLAY, "")
				.parent().children("div#clone").remove();
	}

	private GQuery createEmptyTr() {
		CssSetter rowHeight = CSS.HEIGHT.with(Length.px(getDTable().config()
				.getBodyRowHeight()));
		return $("<tr/>").css(rowHeight);
	}

	private GQuery createEmptyTd() {
		Length marginTop = Length.px(getDTable().config()
				.getBodyCellMarginTop());
		Length marginRight = Length.px(getDTable().config()
				.getBodyCellMarginRight());
		Length marginBottom = Length.px(getDTable().config()
				.getBodyCellMarginBottom());
		Length marginLeft = Length.px(getDTable().config()
				.getBodyCellMarginLeft());

		CssSetter margin = CSS.MARGIN.with(marginTop, marginRight,
				marginBottom, marginLeft);
		CssSetter lineHeight = CSS.LINE_HEIGHT.with(Length.px(getDTable()
				.config().getBodyLineHeight()));
		String clazz = getDTable().config().isEnabledWordWrapInBodyCells() ? "normal"
				: "nowrap";

		return $("<td/>").append(
				$("<div/>").addClass(clazz).css(margin)
						.append($("<span/>").html("&nbsp;").css(lineHeight)));
	}

	private GQuery setTdAlignment(GQuery td, int rowHeight, String colAlign,
			int rowAlign) {
		switch (TextAlign.valueOf(colAlign.toUpperCase())) {
		case LEFT:
			return setTdAlignment(td, rowHeight, 0, rowAlign);
		case RIGHT:
			return setTdAlignment(td, rowHeight, 1, rowAlign);
		case CENTER:
			return setTdAlignment(td, rowHeight, 2, rowAlign);
		default:
			return td;
		}
	}

	private GQuery setTdAlignment(GQuery td, int rowHeight, int colAlign,
			int rowAlign) {
		GQuery div = td.children("div");
		GQuery span = div.children("span");

		// reset span
		span.css(CSS.TOP, "").css(CSS.BOTTOM, "").css(CSS.LEFT, "")
				.css(CSS.RIGHT, "");

		// horizontal alignment
		switch (colAlign) {
		case 0: // left alignment
		case 2: // center align
			span.css(CSS.LEFT.with(Length.px(0)));
			break;
		case 1: // right alignment
			span.css(CSS.RIGHT.with(Length.px(0)));
			break;
		default:
			// nothing to do
			break;
		}

		// vertical alignment
		switch (rowAlign) {
		case 0: // top align
		case 2: // middle align
			span.css(CSS.POSITION.with(Position.ABSOLUTE)).css(
					CSS.TOP.with(Length.px(0)));
			break;
		case 1: // bottom align
			span.css(CSS.POSITION.with(Position.ABSOLUTE)).css(
					CSS.BOTTOM.with(Length.px(0)));
			break;
		default:
			// nothing to do
			break;
		}
		div.css(CSS.POSITION.with(Position.RELATIVE)).css(
				CSS.HEIGHT.with(Length.px(rowHeight)));

		return td.css(CSS.TEXT_ALIGN.with(VDTable.COLUMN_ALIGN[colAlign]));
	}

	private void setDivLeftAlign(GQuery div) {
		div.children("span").css(CSS.LEFT.with(Length.px(0)))
				.css(CSS.RIGHT, "").css(CSS.WIDTH, "");
	}

	private void setDivCenterAlign(GQuery div) {
		div.children("span").css(CSS.WIDTH.with(Length.pct(100)));
	}

	private void setDivMiddleAlign(GQuery div, int top, int bottom) {
		div.css(CSS.MARGIN_TOP.with(Length.px(top)))
				.css(CSS.MARGIN_BOTTOM.with(Length.px(bottom)))
				.css(CSS.HEIGHT, "").css(CSS.POSITION, "").children("span")
				.css(CSS.POSITION, "").css(CSS.TOP, "").css(CSS.BOTTOM, "");
	}

	private void setDivTopAlign(GQuery div, int height) {
		div.css(CSS.HEIGHT.with(Length.px(height))).children("span")
				.css(CSS.TOP.with(Length.px(0))).css(CSS.BOTTOM, "");
	}

	private GQuery setTdWidth(GQuery td, int colWidth) {
		return td.css(CSS.WIDTH.with(Length.px(colWidth)));
	}

	private GQuery createBaseTr(int scrollLeft, int left,
			List<Integer> columns, boolean checkWidth) {
		if (columns != null && columns.size() > 0) {
			int controledWidth = -scrollLeft + left;
			int tableWidth = BrowserInfo.get().getScreenWidth();

			GQuery baseTr = createEmptyTr();
			GQuery baseTd = createEmptyTd();

			int rowHeight = getDTable().config().getBodyRowHeight()
					- getDTable().config().getBodyCellMarginTop()
					- getDTable().config().getBodyCellMarginBottom() - 1;
			for (int column : columns) {
				setTdWidth(
						setTdAlignment(baseTd.clone(), rowHeight,
								columnAlignments.get(column), getDTable()
										.config().getBodyRowAlignments()),
						getDTable().getColumnsWidth().get(column)
								- getDTable().getBrowserOffset()).appendTo(
						baseTr);

				// only display columns which are in display area
				if (checkWidth && !getDTable().config().isEnabledAllColumns()) {
					controledWidth += getDTable().getColumnsWidth().get(column);
					if (controledWidth > tableWidth) {
						break;
					}
				}
			}

			if (!getDTable().config().isEnabledHorizontalLines()) {
				baseTr.children("td").css(
						CSS.BORDER_TOP_COLOR.with(RGBColor.TRANSPARENT));
				baseTr.children("td").css(
						CSS.BORDER_BOTTOM_COLOR.with(RGBColor.TRANSPARENT));
			}

			if (!getDTable().config().isEnabledBodyVerticalLines()) {
				baseTr.children("td").css(
						CSS.BORDER_LEFT_COLOR.with(RGBColor.TRANSPARENT));
				baseTr.children("td").css(
						CSS.BORDER_RIGHT_COLOR.with(RGBColor.TRANSPARENT));
			}

			return baseTr;
		}
		return null;
	}

	private GQuery setTdImage(GQuery div, String value, String id) {
		String[] data = value.split("_:_");
		int width = Integer.parseInt(data[2]);
		int height = Integer.parseInt(data[3]);
		int rowHeight = getDTable().config().getBodyRowHeight()
				- (getDTable().config().getBodyCellMarginTop()
						+ getDTable().config().getBodyCellMarginBottom() + 1);
		float zoom = (float) (rowHeight) / height;

		zoom = zoom > 1 ? 1 : zoom;
		width *= zoom;
		height *= zoom;

		return div
				.css(CSS.HEIGHT.with(Length.px(rowHeight)))
				.children("span")
				.html("")
				.css(CSS.LINE_HEIGHT, "")
				.css(CSS.HEIGHT.with(Length.px(height)))
				.append($("<img/>").id(id)
						.css(CSS.WIDTH.with(Length.px(width)))
						.css(CSS.HEIGHT.with(Length.px(height)))
						.attr("src", data[1]));
	}

	private GQuery setTdValue(GQuery div, String value, String id) {
		div.id(id + ":1").children("span").id(id + ":2");
		if (value == null || value.isEmpty()) {
			return div;
		} else if (value.startsWith("image_:_")) {
			return setTdImage(div, value, id + ":3");
		}
		return div.children("span").html(value);
	}

	private GQuery setTrValues(GQuery tr, final String id,
			final List<String> values, final List<Integer> columns,
			String tableId) {
		return tr.id(id + tableId).children("td").each(new Function() {
			public Object f(Element e, int col) {
				return setTdValue($(e).id(id + columns.get(col))
						.children("div"), values.get(col),
						id + columns.get(col));
			}
		}).parent();
	}

	private List<String> getRowsData(int row, boolean isTableFrozen) {
		List<String> values = new ArrayList<String>();
		for (int column : isTableFrozen ? getDTable().getColumnsFrozen()
				: getDTable().getColumnsVisible()) {
			values.add(tableData.get(column)[row]);
		}
		return values;
	}

	private void writeDataToTable() {
		if (getDTable().config().isEnabledFrozenTable()) {
			$(frozenBodyId).empty();
		}
		$(bodyId).empty();

		long firstRow = getDTable().config().isEnabledPageableTable() ? 0
				: getDTable().getFirstRow();
		long activeRows = tableData.size() > 0 ? tableData.get(getDTable()
				.getColumnsVisible().get(0)).length : 0;
		GQuery baseFrozenTr = createBaseTr(getDTable().getScrollLeftFrozen(),
				0, getDTable().getColumnsFrozen(), false);
		GQuery baseTr = createBaseTr(getDTable().getScrollLeft(), getDTable()
				.getLeft(), getDTable().getColumnsVisible(), !getDTable()
				.isAllColumnsVisible());

		for (int row = 0; row < activeRows; row++) {
			String id = getDTable().getId() + ":" + row + ":";
			String trClass = (firstRow + row) % 2 == 1
					&& getDTable().config().isEnabledOddAndEvenColoringRows() ? "even"
					: "odd";

			if (getDTable().getColumnsFrozen() != null
					&& getDTable().getColumnsFrozen().size() > 0) {
				setTrValues(baseFrozenTr.clone().addClass(trClass), id,
						getRowsData(row, true), getDTable().getColumnsFrozen(),
						"ft").appendTo($(frozenBodyId));
			}
			if (getDTable().getColumnsVisible() != null) {
				setTrValues(baseTr.clone().addClass(trClass), id,
						getRowsData(row, false),
						getDTable().getColumnsVisible(), "t").appendTo(
						$(bodyId));
			}
		}

		if (getDTable().getColumnsFrozen() != null
				&& getDTable().getColumnsFrozen().size() > 0) {
			checkTdHeightsAndAlignments($(frozenBodyId).children("tr"), false);
		}
		checkTdHeightsAndAlignments($(bodyId).children("tr"), false);
	}

	private void activateFrozenRowHighlightEventHandler(boolean enable) {
		if (enable) {
			activateFrozenRowHighlightEventHandler(false);
			frozenRowHighlightEventHandler = $(frozenBodyId).delegate("tr",
					"mouseover", new Function() {
						public void f(Element e) {
							TableEvent.bodyRowMouseOver($(frozenBodyId),
									$(bodyId), $(e), getDTable().config()
											.isEnabledHorizontalLines());
						}
					}).delegate("tr", "mouseout", new Function() {
				public boolean f(Event e) {
					return TableEvent.bodyRowMouseOut($(frozenBodyId),
							$(bodyId), e, getDTable().config()
									.isEnabledHorizontalLines());
				}
			});
		} else if (!enable && frozenRowHighlightEventHandler != null) {
			frozenRowHighlightEventHandler.undelegate("tr", "mouseover")
					.undelegate("tr", "mouseout");
			frozenRowHighlightEventHandler = null;
		}
	}

	private void activateFrozenCellMouseOverEventHandler(boolean enable) {
		if (enable) {
			activateFrozenCellMouseOverEventHandler(false);
			frozenCellHoverEventHandler = $(frozenBodyId).delegate("td",
					"mouseover", new Function() {
						public void f(Element e) {
							TableEvent.cellMouseOver(getDTable(), $(e), null);
							if (getDTable()
									.config()
									.isEnabledExpandedFrozenHeaderCellForActiveColumn()) {
								String[] id = e.getId().split(":");
								GQuery td = $(frozenHeaderId + " tr").children(
										"td [id='" + id[0] + "h:0:" + id[2]
												+ "']");
								TableEvent.cellMouseOver(getDTable(), td, null);
							}
						}
					}).delegate("td", "mouseout", new Function() {
				public boolean f(Event e) {
					return TableEvent.bodyCellMouseOut(frozenHeaderId + " tr",
							e);
				};
			});
		} else if (!enable && frozenCellHoverEventHandler != null) {
			frozenCellHoverEventHandler.undelegate("td", "mouseover")
					.undelegate("td", "mouseout");
			frozenCellHoverEventHandler = null;
		}
	}

	private void activateFrozenCellContextMenuEventHandler(boolean enable) {
		if (enable) {
			activateFrozenCellContextMenuEventHandler(false);
			frozenCellContextMenuEventHandler = $(frozenBodyId).delegate("td",
					"contextmenu", new Function() {
						public boolean f(Event e) {
							return TableEvent.contextMenu(getDTable(), e);
						}
					});
		} else if (!enable && frozenCellContextMenuEventHandler != null) {
			frozenCellContextMenuEventHandler.undelegate("td", "contextmenu");
			frozenCellContextMenuEventHandler = null;
		}
	}

	private void activateRowHighlightEventHandler(boolean enable) {
		if (enable) {
			activateRowHighlightEventHandler(false);
			rowHighlightEventHandler = $(bodyId).delegate("tr", "mouseover",
					new Function() {
						public void f(Element e) {
							TableEvent.bodyRowMouseOver($(bodyId),
									$(frozenBodyId), $(e), getDTable().config()
											.isEnabledHorizontalLines());
						}
					}).delegate("tr", "mouseout", new Function() {
				public boolean f(Event e) {
					return TableEvent.bodyRowMouseOut($(bodyId),
							$(frozenBodyId), e, getDTable().config()
									.isEnabledHorizontalLines());
				}
			});
		} else if (!enable && rowHighlightEventHandler != null) {
			rowHighlightEventHandler.undelegate("tr", "mouseover").undelegate(
					"tr", "mouseout");
			rowHighlightEventHandler = null;
		}
	}

	private void activateCellMouseOverEventHandler(boolean enable) {
		if (enable) {
			activateCellMouseOverEventHandler(false);
			cellMouseOverEventHandler = $(bodyId).delegate("td", "mouseover",
					new Function() {
						public void f(Element e) {
							TableEvent.cellMouseOver(getDTable(), $(e), null);
							if (getDTable()
									.config()
									.isEnabledExpandedHeaderCellForActiveColumn()) {
								String[] id = e.getId().split(":");
								GQuery td = $(headerId + " tr").children(
										"td [id='" + id[0] + "h:0:" + id[2]
												+ "']");
								TableEvent.cellMouseOver(getDTable(), td, null);
							}
						}
					}).delegate("td", "mouseout", new Function() {
				public boolean f(Event e) {
					return TableEvent.bodyCellMouseOut(headerId + " tr", e);
				}
			});
		} else if (!enable && cellMouseOverEventHandler != null) {
			cellMouseOverEventHandler.undelegate("td", "mouseover").undelegate(
					"td", "mouseout");
			cellMouseOverEventHandler = null;
		}
	}

	private void activateCellContextMenuEventHandler(boolean enable) {
		if (enable) {
			activateCellContextMenuEventHandler(false);
			cellContextMenuEventHandler = $(bodyId).delegate("td",
					"contextmenu", new Function() {
						public boolean f(Event e) {
							return TableEvent.contextMenu(getDTable(), e);
						}
					});
		} else if (!enable && cellContextMenuEventHandler != null) {
			cellContextMenuEventHandler.undelegate("td", "contextmenu");
			cellContextMenuEventHandler = null;
		}
	}

	public void set100percentWidth(boolean is100percentWidth) {
		$(bodyId).css("width", is100percentWidth ? "100%" : "0");
	}

	public void checkTdHeightsAndAlignments(GQuery tr,
			final boolean resetAlignments) {
		final int rowHeight = getDTable().config().getBodyRowHeight() - 1;
		final int height = rowHeight
				- getDTable().config().getBodyCellMarginTop()
				- getDTable().config().getBodyCellMarginBottom();
		final int rowAlignments = getDTable().config().getBodyRowAlignments();
		tr.each(new Function() {
			public void f(Element e) {
				if (resetAlignments) {
					$(e).children("td").each(new Function() {
						public void f(Element e) {
							setTdAlignment($(e), height,
									$(e).css(CSS.TEXT_ALIGN), rowAlignments);
						}
					});
				}

				$(e).children("td").each(new Function() {
					public void f(Element e) {
						GQuery div = $(e).children("div");
						GQuery span = div.children("span").css(CSS.WIDTH, "");
						int divWidth = div.width();
						int spanWidth = span.outerWidth();
						int divHeight = div.height();
						int spanHeight = span.outerHeight();

						if (divWidth < spanWidth) {
							setDivLeftAlign(div);
						} else {
							setDivCenterAlign(div);
						}

						if (height < divHeight || divHeight < spanHeight) {
							setDivTopAlign(div, height);
						}

						// middle align
						if (rowAlignments == 2 && spanHeight < divHeight) {
							int top = (rowHeight - spanHeight) / 2;
							int bottom = rowHeight - spanHeight - top;
							setDivMiddleAlign(div, top, bottom);
						}

						// image
						if (span.children("img").length() > 0) {
							// div.css(CSS.HEIGHT.with(Length.px(height)));
						}
					}
				});
			}
		});
	}

	@Override
	public void setIds() {
		if (!bodyId.endsWith("" + getDTable().getId())) {
			frozenHeaderId += getDTable().getId();
			headerId += getDTable().getId();
			frozenBodyId += getDTable().getId();
			bodyId += getDTable().getId();
		}
	}

	public void readUIDL(UIDL uidl) {
		List<Integer> frozenAndVisibleColumns = new ArrayList<Integer>();
		frozenAndVisibleColumns.addAll(getDTable().getColumnsFrozen());
		frozenAndVisibleColumns.addAll(getDTable().getColumnsVisible());
		Collections.sort(frozenAndVisibleColumns);

		if (uidl.hasAttribute("bodyColumnsAlignment")) {
			columnAlignments.clear();
			int[] alignments = uidl
					.getIntArrayAttribute("bodyColumnsAlignment");
			for (int column = 0; column < frozenAndVisibleColumns.size(); column++) {
				columnAlignments.put(frozenAndVisibleColumns.get(column),
						alignments[column]);
			}
		}

		tableData.clear();
		for (int column : frozenAndVisibleColumns) {
			if (uidl.hasAttribute("" + column)) {
				tableData
						.put(column, uidl.getStringArrayAttribute("" + column));
			}
		}
	}

	@Override
	public void load() {
		setIds();
		enableEventsHandler();
	}

	@Override
	public void unload() {
		disableEventsHandler();
	}

	public void show() {
		writeDataToTable();
		getDTable().filter().markFilter();
		getDTable().checkTableScrollers($(frozenHeaderId), $(frozenBodyId),
				$(headerId), $(bodyId));
	}

	@Override
	public void enableEventsHandler() {
		activateCellContextMenuEventHandler(true);
		activateRowHighlightEventHandler(getDTable().config()
				.isEnabledHighlightActiveRow());
		activateCellMouseOverEventHandler(getDTable().config()
				.isEnabledExpandedBodyCell()
				|| getDTable().config()
						.isEnabledExpandedBodyCellAlwaysVisible());

		if (getDTable().config().isEnabledFrozenTable()) {
			activateFrozenCellContextMenuEventHandler(true);
			activateFrozenRowHighlightEventHandler(getDTable().config()
					.isEnabledHighlightActiveRow());
			activateFrozenCellMouseOverEventHandler(getDTable().config()
					.isEnabledExpandedBodyCell()
					|| getDTable().config()
							.isEnabledExpandedBodyCellAlwaysVisible());
		}
	}

	@Override
	public void disableEventsHandler() {
		removeExpandedCells();
		activateFrozenRowHighlightEventHandler(false);
		activateFrozenCellMouseOverEventHandler(false);
		activateFrozenCellContextMenuEventHandler(false);
		activateRowHighlightEventHandler(false);
		activateCellMouseOverEventHandler(false);
		activateCellContextMenuEventHandler(false);
	}

}
