/**
 * Copyright (C) 2012  Dušan Vejnovič  <vaadin@dussan.org>
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 * http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */
package org.dussan.vaadin.dtable.config;

import java.io.Serializable;
import java.text.Format;
import java.util.ArrayList;
import java.util.Arrays;
import java.util.Collections;
import java.util.HashMap;
import java.util.LinkedHashSet;
import java.util.List;
import java.util.Locale;
import java.util.Map;
import java.util.Set;

import org.apache.commons.lang3.ArrayUtils;
import org.dussan.vaadin.dtable.config.formatting.Formatting;
import org.dussan.vaadin.dtable.config.formatting.FormattingHelper;
import org.dussan.vaadin.dtable.config.freeze.ColumnFreeze;
import org.dussan.vaadin.dtable.config.freeze.ColumnUnfreeze;
import org.dussan.vaadin.dtable.config.hide.ColumnHide;
import org.dussan.vaadin.dtable.config.hide.ColumnUnhide;
import org.dussan.vaadin.dtable.config.tablesection.alignments.ColumnAlign;
import org.dussan.vaadin.dtable.config.tablesection.alignments.RowAlign;
import org.dussan.vaadin.dtable.config.tablesection.dimensions.ColumnWidth;

public class TableConfig implements Serializable, ColumnWidth, ColumnFreeze,
		ColumnUnfreeze, ColumnHide, ColumnUnhide, FormattingHelper {

	private static final long serialVersionUID = -4562404019696839838L;
	public static final int DEFAULT_TABLE_WIDTH = 4000;
	public static final int DEFAULT_TABLE_HEIGHT = 300;
	public static final int DEFAULT_TABLE_FROZEN_MINIMAL_WIDTH = 150;
	public static final int DEFAULT_TABLE_MINIMAL_WIDTH = 300;
	public static final int DEFAULT_CONTEXT_MENU_DEPTH_FOR_FILTERED_FROZEN_TABLE = 3;
	public static final int DEFAULT_CONTEXT_MENU_DEPTH_FOR_FILTERED_TABLE = 3;
	public static final int DEFAULT_PAGER_PAGES = 15;
	public static final int DEFAULT_PAGE_SIZE = 15;
	public static final FilterInfoPosition DEFAULT_FILTER_INFO_POSITION = FilterInfoPosition.BOTTOM;
	public static final TableToolsPosition DEFAULT_TABLE_TOOLS_POSITION = TableToolsPosition.TOP_RIGHT;

	public static final int DEFAULT_PAGE_REFRESH = 500;
	public static final int DEFAULT_PAGE_RESIZING_ACTION_DELAY = 500;
	public static final int SCROLLBAR_ACTION_DELAY = 600;
	public static final int ACTION_TRAY_DELAY = 300;
	private static final int MINIMAL_DELAY = 100;

	public static final ColumnAlign DEFAULT_HEADER_COLUMN_ALIGNMENT = ColumnAlign.CENTER;
	public static final RowAlign DEFAULT_HEADER_ROW_ALIGNMENT = RowAlign.MIDDLE;
	public static final int DEFAULT_HEADER_ROW_HEIGHT = 30;
	public static final int DEFAULT_HEADER_LINE_HEIGHT = 22;
	public static final int DEFAULT_HEADER_TOP_BOTTOM_MARGINS = 3;
	public static final int DEFAULT_HEADER_LEFT_RIGHT_MARGINS = 6;

	public static final ColumnAlign DEFAULT_BODY_COLUMN_ALIGNMENT = ColumnAlign.LEFT;
	public static final RowAlign DEFAULT_BODY_ROW_ALIGNMENT = RowAlign.MIDDLE;
	public static final int DEFAULT_BODY_ROW_HEIGHT = 24;
	public static final int DEFAULT_BODY_LINE_HEIGHT = 20;
	public static final int DEFAULT_BODY_TOP_BOTTOM_MARGINS = 2;
	public static final int DEFAULT_BODY_LEFT_RIGHT_MARGINS = 4;

	public static final float DEFAULT_LOADING_MESSAGE_OPACITY = 0.5f;
	public static final int DEFAULT_LOADING_MESSAGE_SHOW_DELAY = 100;
	public static final int DEFAULT_LOADING_MESSAGE_HIDE_DELAY = 300;

	private HeaderConfig headerConfig = new HeaderConfig();
	private BodyConfig bodyConfig = new BodyConfig();
	private Locale locale = null;

	private int pageResizingActionDelay = DEFAULT_PAGE_RESIZING_ACTION_DELAY;
	private int scrollbarActionDelay = SCROLLBAR_ACTION_DELAY;
	private int actionTrayDelay = ACTION_TRAY_DELAY;

	private int allColumns = 0;
	private int tableWidth = DEFAULT_TABLE_WIDTH;
	private int tableHeight = DEFAULT_TABLE_HEIGHT;
	private int tableFrozenMinimalWidth = DEFAULT_TABLE_FROZEN_MINIMAL_WIDTH;
	private int tableMinimalWidth = DEFAULT_TABLE_MINIMAL_WIDTH;
	private int contextMenuDepthForFilteredFrozenTable = DEFAULT_CONTEXT_MENU_DEPTH_FOR_FILTERED_FROZEN_TABLE;
	private int contextMenuDepthForFilteredTable = DEFAULT_CONTEXT_MENU_DEPTH_FOR_FILTERED_TABLE;
	private int pagerPages = DEFAULT_PAGER_PAGES;
	private int pageSize = DEFAULT_PAGE_SIZE;
	private TableToolsPosition tableToolsPosition = DEFAULT_TABLE_TOOLS_POSITION;
	private FilterInfoPosition filterInfoPosition = DEFAULT_FILTER_INFO_POSITION;

	private boolean checkColumnsWidth = true;
	private int columnDefaultWidth = ColumnWidth.DEFAULT_COLUMN_WIDTH;
	private Map<Integer, Integer> columnsWidth = new HashMap<Integer, Integer>();
	private List<String> columnFreezeEnable = new ArrayList<String>();
	private List<String> columnUnfreezeEnable = new ArrayList<String>();
	private List<String> columnHideEnable = new ArrayList<String>();
	private List<String> columnUnhideEnable = new ArrayList<String>();
	private List<Formatting> formattingColumns = new ArrayList<Formatting>();

	private boolean enableAutoPrepareData = false;
	private boolean enableAutoPrepareDataWithoutHiddenColumns = false;
	private boolean enableAutoPrepareDataWithoutRemovedColumns = false;

	private boolean enableAllColumns = false;
	private boolean enableAllRows = false;
	private boolean enableFixedTableHeight = false;
	private boolean enablePageableTable = false;
	private boolean enableHorizontalLines = false;
	private boolean enableHighlightActiveRow = false;
	private boolean enableOddAndEvenColoringRows = false;

	private boolean enableQuickSearch = false;
	private boolean enableQuickSearchFromatting = false;
	private boolean enableQuickSearchImmediate = false;
	private boolean enableQuickSearchVisibleAtStartup = false;
	private boolean enableQuickSearchHide = false;
	private boolean enableQuickSearchClearOnNoNewData = false;

	private boolean enableSearch = false;
	private boolean enableSearchFromatting = false;
	private boolean enableSearchImmediate = false;
	private boolean enableSearchVisibleAtStartup = false;
	private boolean enableSearchHide = false;
	private boolean enableSearchClearOnNoNewData = false;

	private boolean enableNoNewDataMessage = false;
	private boolean enableLoadingMessage = false;
	private boolean enableColumnNumberInLoadingMessageBox = false;

	private boolean enableOrderedFrozenTable = false;
	private boolean enableOrderedTable = false;

	private boolean enableFilteredFrozenTable = false;
	private boolean enableFilteredTable = false;
	private boolean enableFilterInfo = false;
	private boolean enableContextMenuForFilteredFrozenTable = false;
	private boolean enableContextMenuForFilteredTable = false;

	private boolean enableFrozenTable = false;
	private boolean enableFrozenTableVisible = false;
	private boolean enableHideFrozenTable = false;
	private boolean enableEmptyFrozenTable = false;
	private boolean enableResizeFrozenTable = false;
	private boolean enableFreezeUnfreezeColumns = false;
	private boolean enableFreezeAnyColumns = false;
	private boolean enableUnfreezeAnyColumns = false;
	private boolean enableHideUnhideColumns = false;
	private boolean enableHideAnyColumns = false;
	private boolean enableUnhideAnyColumns = false;
	private boolean enableNewDataWhenHideUnhideColumn = false;

	private float loadingMessageOpacity = DEFAULT_LOADING_MESSAGE_OPACITY;
	private int loadingMessageShowDelay = DEFAULT_LOADING_MESSAGE_SHOW_DELAY;
	private int loadingMessageHideDelay = DEFAULT_LOADING_MESSAGE_HIDE_DELAY;

	public TableConfig() {
		header().setDefaultColumnAlignment(DEFAULT_HEADER_COLUMN_ALIGNMENT);
		header().setRowAlignment(DEFAULT_HEADER_ROW_ALIGNMENT);
		header().setRowHeight(DEFAULT_HEADER_ROW_HEIGHT);
		header().setLineHeight(DEFAULT_HEADER_LINE_HEIGHT);
		header().setMargins(DEFAULT_HEADER_TOP_BOTTOM_MARGINS,
				DEFAULT_HEADER_LEFT_RIGHT_MARGINS);

		body().setDefaultColumnAlignment(DEFAULT_BODY_COLUMN_ALIGNMENT);
		body().setRowAlignment(DEFAULT_BODY_ROW_ALIGNMENT);
		body().setRowHeight(DEFAULT_BODY_ROW_HEIGHT);
		body().setLineHeight(DEFAULT_BODY_LINE_HEIGHT);
		body().setMargins(DEFAULT_BODY_TOP_BOTTOM_MARGINS,
				DEFAULT_BODY_LEFT_RIGHT_MARGINS);
	}

	private void checkColumnsWidth() {
		if (checkColumnsWidth && allColumns > 0) {
			// check columns number
			List<Integer> columns = Arrays.asList(columnsWidth.keySet()
					.toArray(new Integer[] {}));
			Collections.sort(columns);
			if (!columns.isEmpty()
					&& !(columns.get(0) >= 0 && columns.get(columns.size() - 1) < allColumns)) {
				Map<Integer, Integer> widths = new HashMap<Integer, Integer>();
				for (int column : columns) {
					if (column >= 0 && column < allColumns) {
						widths.put(column, columnsWidth.get(column));
					}
				}
				columnsWidth = new HashMap<Integer, Integer>(widths);
			}

			// add missing columns width
			if (columnsWidth.size() < allColumns) {
				for (int column = 0; column < allColumns; column++) {
					if (!columnsWidth.containsKey(column)) {
						columnsWidth.put(column, columnDefaultWidth);
					}
				}
			}
			checkColumnsWidth = false;
		}
	}

	public void setAllColumns(int allColumns) {
		this.allColumns = allColumns > 0 ? allColumns : this.allColumns;
		header().setAllColumns(this.allColumns);
		body().setAllColumns(this.allColumns);
	}

	public HeaderConfig header() {
		return headerConfig;
	}

	public BodyConfig body() {
		return bodyConfig;
	}

	public Locale getLocale() {
		return locale;
	}

	public void setLocale(Locale locale) {
		this.locale = locale;
	}

	public int getPageResizingActionDelay() {
		return pageResizingActionDelay;
	}

	public void setPageResizingActionDelay(int pageResizingActionDelay) {
		this.pageResizingActionDelay = pageResizingActionDelay >= MINIMAL_DELAY ? pageResizingActionDelay
				: this.pageResizingActionDelay;
	}

	public int getScrollbarActionDelay() {
		return scrollbarActionDelay;
	}

	public void setScrollbarActionDelay(int scrollbarActionDelay) {
		this.scrollbarActionDelay = scrollbarActionDelay >= MINIMAL_DELAY ? scrollbarActionDelay
				: this.scrollbarActionDelay;
	}

	public int getActionTrayDelay() {
		return actionTrayDelay;
	}

	public void setActionTrayDelay(int actionTrayDelay) {
		this.actionTrayDelay = actionTrayDelay >= MINIMAL_DELAY ? actionTrayDelay
				: this.actionTrayDelay;
	}

	public int getTableWidth() {
		return tableWidth;
	}

	public void setTableWidth(int width) {
		tableWidth = width > 0 ? width : tableWidth;
	}

	public int getTableHeight() {
		return tableHeight;
	}

	public void setTableHeight(int height) {
		// table height minus 2 (2 = borders)
		tableHeight = height > 0 ? (height - 2) : DEFAULT_TABLE_HEIGHT;
	}

	public int getTableFrozenMinimalWidth() {
		return tableFrozenMinimalWidth;
	}

	public void setTableFrozenMinimalWidth(int width) {
		tableFrozenMinimalWidth = width > 0 ? width : tableFrozenMinimalWidth;
	}

	public int getTableMinimalWidth() {
		return tableMinimalWidth;
	}

	public void setTableMinimalWidth(int width) {
		tableMinimalWidth = width > 0 ? width : tableMinimalWidth;
	}

	public int getContextMenuDepthForFilteredFrozenTable() {
		return contextMenuDepthForFilteredFrozenTable;
	}

	public void setContextMenuDepthForFilteredFrozenTable(
			int contextMenuDepthForFilteredFrozenTable) {
		this.contextMenuDepthForFilteredFrozenTable = Math.max(
				DEFAULT_CONTEXT_MENU_DEPTH_FOR_FILTERED_FROZEN_TABLE,
				contextMenuDepthForFilteredFrozenTable);
	}

	public int getContextMenuDepthForFilteredTable() {
		return contextMenuDepthForFilteredTable;
	}

	public void setContextMenuDepthForFilteredTable(
			int contextMenuDepthForFilteredTable) {
		this.contextMenuDepthForFilteredTable = Math.max(
				DEFAULT_CONTEXT_MENU_DEPTH_FOR_FILTERED_TABLE,
				contextMenuDepthForFilteredTable);
	}

	public int getPagerPages() {
		return pagerPages;
	}

	public void setPagerPages(int pages) {
		pagerPages = pages > 0 ? pages : pagerPages;
	}

	public int getPageSize() {
		pageSize = !isEnabledPageableTable() ? (getTableHeight()
				/ body().getRowHeight() + 2) : pageSize;
		return pageSize;
	}

	public void setPageSize(int size) {
		pageSize = size > 0 ? size : pageSize;
	}

	public String getTableToolsPosition() {
		return tableToolsPosition.getPosition();
	}

	public void setTableToolsPosition(TableToolsPosition tableToolsPosition) {
		this.tableToolsPosition = tableToolsPosition;
	}

	public String getFilterInfoPosition() {
		return filterInfoPosition.getPosition();
	}

	public void setFilterInfoPosition(FilterInfoPosition filterInfoPosition) {
		this.filterInfoPosition = filterInfoPosition;
	}

	@Override
	public int getColumnDefaultWidth() {
		return columnDefaultWidth;
	}

	@Override
	public void setColumnDefaultWidth(int width) {
		columnDefaultWidth = width > 0 ? width : columnDefaultWidth;
	}

	@Override
	public Integer getColumnWidth(int column) {
		return (columnsWidth.containsKey(column) ? columnsWidth.get(column)
				: columnDefaultWidth);
	}

	@Override
	public void setColumnWidth(int column, int width) {
		checkColumnsWidth = true;
		columnsWidth.put(column, width > 0 ? width : columnDefaultWidth);
	}

	@Override
	public Map<Integer, Integer> getColumnWidthsMap() {
		checkColumnsWidth();
		return columnsWidth;
	}

	@Override
	public int[] getColumnsWidth(int[] columns) {
		checkColumnsWidth();
		List<Integer> widths = new ArrayList<Integer>();
		for (int column : columns) {
			widths.add(columnsWidth.get(column));
		}
		return ArrayUtils.toPrimitive(widths.toArray(new Integer[] {}));
	}

	@Override
	public int[] getColumnsWidth() {
		checkColumnsWidth();
		int[] widths = new int[allColumns];
		for (int column = 0; column < allColumns; column++) {
			widths[column] = columnsWidth.get(column);
		}
		return widths;
	}

	@Override
	public void setColumnsWidth(int... widths) {
		checkColumnsWidth = true;
		for (int column = 0; column < widths.length; column++) {
			setColumnWidth(column, widths[column]);
		}
	}

	@Override
	public void setColumnsSameWidth(int width, int... columns) {
		checkColumnsWidth = true;
		for (int column : columns) {
			setColumnWidth(column, width);
		}
	}

	@Override
	public void setColumnsWidth(Map<Integer, Integer> widths) {
		checkColumnsWidth = true;
		for (int column : widths.keySet()) {
			setColumnWidth(column, widths.get(column));
		}
	}

	public boolean isEnabledAutoPrepareData() {
		return enableAutoPrepareData;
	}

	public void enableAutoPrepareData(boolean enableAutoPrepareData) {
		this.enableAutoPrepareData = enableAutoPrepareData;
	}

	public boolean isEnabledAutoPrepareDataWithoutHiddenColumns() {
		return enableAutoPrepareDataWithoutHiddenColumns;
	}

	public void enableAutoPrepareDataWithoutHiddenColumns(
			boolean enableAutoPrepareDataWithoutHiddenColumns) {
		this.enableAutoPrepareDataWithoutHiddenColumns = enableAutoPrepareDataWithoutHiddenColumns;
	}

	public boolean isEnabledAutoPrepareDataWithoutRemovedColumns() {
		return enableAutoPrepareDataWithoutRemovedColumns;
	}

	public void enableAutoPrepareDataWithoutRemovedColumns(
			boolean enableAutoPrepareDataWithoutRemovedColumns) {
		this.enableAutoPrepareDataWithoutRemovedColumns = enableAutoPrepareDataWithoutRemovedColumns;
	}

	public boolean isEnabledAllColumns() {
		return enableAllColumns;
	}

	public void enableAllColumns(boolean enableAllColumns) {
		this.enableAllColumns = enableAllColumns;
	}

	public boolean isEnabledAllRows() {
		return enableAllRows;
	}

	public void enableAllRows(boolean enableAllRows) {
		this.enableAllRows = enableAllRows;
	}

	public boolean isEnabledLazyLoading() {
		return !isEnabledAllRows();
	}

	public void enableLazyLoading(boolean enableLazyLoading) {
		enableAllRows(!enableLazyLoading);
	}

	public boolean isEnabledFixedTableHeight() {
		return enableFixedTableHeight;
	}

	public void enableFixedTableHeight(boolean enableFixedTableHeight) {
		this.enableFixedTableHeight = enableFixedTableHeight;
	}

	public boolean isEnabledPageableTable() {
		return enablePageableTable;
	}

	public void enablePageableTable(boolean enablePageableTable) {
		this.enablePageableTable = enablePageableTable;
	}

	public boolean isEnabledScrollableTable() {
		return !enablePageableTable;
	}

	public void enableScrollableTable(boolean enableScrollableTable) {
		this.enablePageableTable = !enableScrollableTable;
	}

	public boolean isEnabledHorizontalLines() {
		return enableHorizontalLines;
	}

	public void enableHorizontalLines(boolean enableHorizontalLines) {
		this.enableHorizontalLines = enableHorizontalLines;
	}

	public boolean isEnabledHighlightActiveRow() {
		return enableHighlightActiveRow;
	}

	public void enableHighlightActiveRow(boolean enableHighlightActiveRow) {
		this.enableHighlightActiveRow = enableHighlightActiveRow;
	}

	public boolean isEnabledOddAndEvenColoringRows() {
		return enableOddAndEvenColoringRows;
	}

	public void enableOddAndEvenColoringRows(
			boolean enableOddAndEvenColoringRows) {
		this.enableOddAndEvenColoringRows = enableOddAndEvenColoringRows;
	}

	public boolean isEnabledQuickSearch() {
		return enableQuickSearch;
	}

	public void enableQuickSearch(boolean enableQuickSearch) {
		this.enableQuickSearch = enableQuickSearch;
	}

	public boolean isEnabledQuickSearchFromatting() {
		return enableQuickSearchFromatting;
	}

	public void enableQuickSearchFromatting(boolean enableQuickSearchFromatting) {
		this.enableQuickSearchFromatting = enableQuickSearchFromatting;
	}

	public boolean isEnabledQuickSearchImmediate() {
		return enableQuickSearchImmediate;
	}

	public void enableQuickSearchImmediate(boolean enableQuickSearchImmediate) {
		this.enableQuickSearchImmediate = enableQuickSearchImmediate;
	}

	public boolean isEnabledQuickSearchVisibleAtStartup() {
		return enableQuickSearchVisibleAtStartup;
	}

	public void enableQuickSearchVisibleAtStartup(
			boolean enableQuickSearchVisibleAtStartup) {
		this.enableQuickSearchVisibleAtStartup = enableQuickSearchVisibleAtStartup;
	}

	public boolean isEnabledQuickSearchHide() {
		return enableQuickSearchHide;
	}

	public void enableQuickSearchHide(boolean enableQuickSearchHide) {
		this.enableQuickSearchHide = enableQuickSearchHide;
	}

	public boolean isEnabledQuickSearchClearOnNoNewData() {
		return enableQuickSearchClearOnNoNewData;
	}

	public void enableQuickSearchClearOnNoNewData(
			boolean enableQuickSearchClearOnNoNewData) {
		this.enableQuickSearchClearOnNoNewData = enableQuickSearchClearOnNoNewData;
	}

	public boolean isEnabledSearch() {
		return enableSearch;
	}

	public void enableSearch(boolean enableSearch) {
		this.enableSearch = enableSearch;
	}

	public boolean isEnabledSearchFromatting() {
		return enableSearchFromatting;
	}

	public void enableSearchFromatting(boolean enableSearchFromatting) {
		this.enableSearchFromatting = enableSearchFromatting;
	}

	public boolean isEnabledSearchImmediate() {
		return enableSearchImmediate;
	}

	public void enableSearchImmediate(boolean enableSearchImmediate) {
		this.enableSearchImmediate = enableSearchImmediate;
	}

	public boolean isEnabledSearchVisibleAtStartup() {
		return enableSearchVisibleAtStartup;
	}

	public void enableSearchVisibleAtStartup(
			boolean enableSearchVisibleAtStartup) {
		this.enableSearchVisibleAtStartup = enableSearchVisibleAtStartup;
	}

	public boolean isEnabledSearchHide() {
		return enableSearchHide;
	}

	public void enableSearchHide(boolean enableSearchHide) {
		this.enableSearchHide = enableSearchHide;
	}

	public boolean isEnabledSearchClearOnNoNewData() {
		return enableSearchClearOnNoNewData;
	}

	public void enableSearchClearOnNoNewData(
			boolean enableSearchClearOnNoNewData) {
		this.enableSearchClearOnNoNewData = enableSearchClearOnNoNewData;
	}

	public boolean isEnabledNoNewDataMessage() {
		return enableNoNewDataMessage;
	}

	public void enableNoNewDataMessage(boolean enableNoNewDataMessage) {
		this.enableNoNewDataMessage = enableNoNewDataMessage;
	}

	public boolean isEnabledLoadingMessage() {
		return enableLoadingMessage;
	}

	public void enableLoadingMessage(boolean enableLoadingMessage) {
		this.enableLoadingMessage = enableLoadingMessage;
	}

	public boolean isEnabledColumnNumberInLoadingMessageBox() {
		return enableColumnNumberInLoadingMessageBox;
	}

	public void enableColumnNumberInLoadingMessageBox(
			boolean enableColumnNumberInLoadingMessageBox) {
		this.enableColumnNumberInLoadingMessageBox = enableColumnNumberInLoadingMessageBox;
	}

	public boolean isEnabledOrderedFrozenTable() {
		return enableOrderedFrozenTable;
	}

	public void enableOrderedFrozenTable(boolean enableOrderedFrozenTable) {
		this.enableOrderedFrozenTable = enableOrderedFrozenTable;
	}

	public boolean isEnabledOrderedTable() {
		return enableOrderedTable;
	}

	public void enableOrderedTable(boolean enableOrderedTable) {
		this.enableOrderedTable = enableOrderedTable;
	}

	public boolean isEnabledFilteredFrozenTable() {
		return enableFilteredFrozenTable;
	}

	public void enableFilteredFrozenTable(boolean enableFilteredFrozenTable) {
		this.enableFilteredFrozenTable = enableFilteredFrozenTable;
	}

	public boolean isEnabledFilteredTable() {
		return enableFilteredTable;
	}

	public void enableFilteredTable(boolean enableFilteredTable) {
		this.enableFilteredTable = enableFilteredTable;
	}

	public boolean isEnabledContextMenuForFilteredFrozenTable() {
		return enableContextMenuForFilteredFrozenTable;
	}

	public void enableContextMenuForFilteredFrozenTable(
			boolean enableContextMenuForFilteredFrozenTable) {
		this.enableContextMenuForFilteredFrozenTable = enableContextMenuForFilteredFrozenTable;
	}

	public boolean isEnabledContextMenuForFilteredTable() {
		return enableContextMenuForFilteredTable;
	}

	public void enableContextMenuForFilteredTable(
			boolean enableContextMenuForFilteredTable) {
		this.enableContextMenuForFilteredTable = enableContextMenuForFilteredTable;
	}

	public boolean isEnabledFilterInfo() {
		return enableFilterInfo;
	}

	public void enableFilterInfo(boolean enableFilterInfo) {
		this.enableFilterInfo = enableFilterInfo;
	}

	public boolean isEnabledFrozenTable() {
		return enableFrozenTable;
	}

	public void enableFrozenTable(boolean enableFrozenTable) {
		this.enableFrozenTable = enableFrozenTable;
	}

	public boolean isEnabledFrozenTableVisible() {
		return enableFrozenTableVisible;
	}

	public void enableFrozenTableVisible(boolean enableFrozenTableVisible) {
		this.enableFrozenTableVisible = enableFrozenTableVisible;
	}

	public boolean isEnabledHideFrozenTable() {
		return enableHideFrozenTable;
	}

	public void enableHideFrozenTable(boolean enableHideFrozenTable) {
		this.enableHideFrozenTable = enableHideFrozenTable;
	}

	public boolean isEnabledEmptyFrozenTable() {
		return enableEmptyFrozenTable;
	}

	public void enableEmptyFrozenTable(boolean enableEmptyFrozenTable) {
		this.enableEmptyFrozenTable = enableEmptyFrozenTable;
	}

	public boolean isEnabledResizeFrozenTable() {
		return enableResizeFrozenTable;
	}

	public void enableResizeFrozenTable(boolean enableResizeFrozenTable) {
		this.enableResizeFrozenTable = enableResizeFrozenTable;
	}

	public boolean isEnabledFreezeUnfreezeColumns() {
		return enableFreezeUnfreezeColumns;
	}

	public void enableFreezeUnfreezeColumns(boolean enableFreezeUnfreezeColumns) {
		this.enableFreezeUnfreezeColumns = enableFreezeUnfreezeColumns;
	}

	public boolean isEnabledFreezeAnyColumns() {
		return enableFreezeAnyColumns;
	}

	public void enableFreezeAnyColumns(boolean enableFreezeAnyColumns) {
		this.enableFreezeAnyColumns = enableFreezeAnyColumns;
	}

	public boolean isEnabledUnfreezeAnyColumns() {
		return enableUnfreezeAnyColumns;
	}

	public void enableUnfreezeAnyColumns(boolean enableUnfreezeAnyColumns) {
		this.enableUnfreezeAnyColumns = enableUnfreezeAnyColumns;
	}

	public boolean isEnabledHideUnhideColumns() {
		return enableHideUnhideColumns;
	}

	public void enableHideUnhideColumns(boolean enableHideUnhideColumns) {
		this.enableHideUnhideColumns = enableHideUnhideColumns;
	}

	public boolean isEnabledHideAnyColumns() {
		return enableHideAnyColumns;
	}

	public void enableHideAnyColumns(boolean enableHideAnyColumns) {
		this.enableHideAnyColumns = enableHideAnyColumns;
	}

	public boolean isEnabledUnhideAnyColumns() {
		return enableUnhideAnyColumns;
	}

	public void enableUnhideAnyColumns(boolean enableUnhideAnyColumns) {
		this.enableUnhideAnyColumns = enableUnhideAnyColumns;
	}

	public boolean isEnabledNewDataWhenHideUnhideColumn() {
		return enableNewDataWhenHideUnhideColumn;
	}

	public void enableNewDataWhenHideUnhideColumn(
			boolean enableNewDataWhenHideUnhideColumn) {
		this.enableNewDataWhenHideUnhideColumn = enableNewDataWhenHideUnhideColumn;
	}

	public float getLoadingMessageOpacity() {
		return loadingMessageOpacity;
	}

	public void setLoadingMessageOpacity(float loadingMessageOpacity) {
		this.loadingMessageOpacity = loadingMessageOpacity > 0 ? loadingMessageOpacity
				: DEFAULT_LOADING_MESSAGE_OPACITY;
	}

	public void setLoadingMessageDelay(int showDelay, int hideDelay) {
		if (showDelay >= MINIMAL_DELAY && hideDelay >= MINIMAL_DELAY) {
			loadingMessageShowDelay = showDelay;
			loadingMessageHideDelay = hideDelay;
		}
	}

	public int getLoadingMessageShowDelay() {
		return loadingMessageShowDelay;
	}

	public void setLoadingMessageShowDelay(int loadingMessageShowDelay) {
		this.loadingMessageShowDelay = loadingMessageShowDelay >= MINIMAL_DELAY ? loadingMessageShowDelay
				: DEFAULT_LOADING_MESSAGE_SHOW_DELAY;
	}

	public int getLoadingMessageHideDelay() {
		return loadingMessageHideDelay;
	}

	public void setLoadingMessageHideDelay(int loadingMessageHideDelay) {
		this.loadingMessageHideDelay = loadingMessageHideDelay >= MINIMAL_DELAY ? loadingMessageHideDelay
				: DEFAULT_LOADING_MESSAGE_HIDE_DELAY;
	}

	private void checkColumnsFreezeEnable() {
		// check column number
		for (int column = columnFreezeEnable.size() - 1; column >= 0; column--) {
			int col = Integer.parseInt(columnFreezeEnable.get(column)
					.substring(1));
			if (!(col >= 0 && col < allColumns)) {
				columnFreezeEnable.remove(column);
			}
		}
		// removed duplicated values
		if (!columnFreezeEnable.isEmpty()) {
			Set<String> tempColumns = new LinkedHashSet<String>(
					columnFreezeEnable);
			columnFreezeEnable = new ArrayList<String>(tempColumns);
		}
	}

	@Override
	public boolean canColumnFreeze(int column) {
		if (columnFreezeEnable.contains("-" + column)) {
			return false;
		} else if (columnFreezeEnable.contains("+" + column)) {
			return true;
		}
		return enableFreezeAnyColumns;
	}

	@Override
	public void enableColumnFreeze(boolean enableFreeze, int column) {
		try {
			columnFreezeEnable.remove("+" + column);
			columnFreezeEnable.remove("-" + column);
		} catch (Exception e) {
		}
		columnFreezeEnable.add((enableFreeze ? "+" : "-") + column);
	}

	@Override
	public boolean isEnabledColumnsFreezeEmpty() {
		return !(enableFreezeAnyColumns && !columnFreezeEnable.isEmpty());
	}

	@Override
	public String[] getColumnsFreezeEnabled(int[] columns) {
		if (enableFreezeAnyColumns || !columnFreezeEnable.isEmpty()) {
			checkColumnsFreezeEnable();
			List<Integer> cols = Arrays.asList(ArrayUtils.toObject(columns));
			List<String> tempCols = new ArrayList<String>();
			for (String column : columnFreezeEnable) {
				if (cols.contains(Integer.parseInt(column.substring(1)))) {
					tempCols.add(column);
				}
			}
			return tempCols.toArray(new String[] {});
		}
		return null;
	}

	@Override
	public String[] getColumnsFreezeEnabled() {
		if (enableFreezeAnyColumns && !columnFreezeEnable.isEmpty()) {
			checkColumnsFreezeEnable();
			return columnFreezeEnable.toArray(new String[] {});
		}
		return null;
	}

	@Override
	public void enableColumnsFreeze(boolean enableFreeze, int... columns) {
		for (int column : columns) {
			enableColumnFreeze(enableFreeze, column);
		}
	}

	private void checkColumnsUnfreezeEnable() {
		// check column number
		for (int column = columnUnfreezeEnable.size() - 1; column >= 0; column--) {
			int col = Integer.parseInt(columnUnfreezeEnable.get(column)
					.substring(1));
			if (!(col >= 0 && col < allColumns)) {
				columnUnfreezeEnable.remove(column);
			}
		}
		// removed duplicated values
		if (!columnUnfreezeEnable.isEmpty()) {
			Set<String> tempColumns = new LinkedHashSet<String>(
					columnUnfreezeEnable);
			columnUnfreezeEnable = new ArrayList<String>(tempColumns);
		}
	}

	@Override
	public boolean canColumnUnfreeze(int column) {
		if (columnUnfreezeEnable.contains("-" + column)) {
			return false;
		} else if (columnUnfreezeEnable.contains("+" + column)) {
			return true;
		}
		return enableUnfreezeAnyColumns;
	}

	@Override
	public void enableColumnUnfreeze(boolean enableUnfreeze, int column) {
		try {
			columnUnfreezeEnable.remove("+" + column);
			columnUnfreezeEnable.remove("-" + column);
		} catch (Exception e) {
		}
		columnUnfreezeEnable.add((enableUnfreeze ? "+" : "-") + column);
	}

	@Override
	public boolean isEnabledColumnsUnfreezeEmpty() {
		return !(enableUnfreezeAnyColumns && !columnUnfreezeEnable.isEmpty());
	}

	@Override
	public String[] getColumnsUnfreezeEnabled(int[] columns) {
		if (enableUnfreezeAnyColumns || !columnUnfreezeEnable.isEmpty()) {
			checkColumnsUnfreezeEnable();
			List<Integer> cols = Arrays.asList(ArrayUtils.toObject(columns));
			List<String> tempCols = new ArrayList<String>();
			for (String column : columnUnfreezeEnable) {
				if (cols.contains(Integer.parseInt(column.substring(1)))) {
					tempCols.add(column);
				}
			}
			return tempCols.toArray(new String[] {});
		}
		return null;
	}

	@Override
	public String[] getColumnsUnfreezeEnabled() {
		if (enableUnfreezeAnyColumns && !columnUnfreezeEnable.isEmpty()) {
			checkColumnsUnfreezeEnable();
			return columnUnfreezeEnable.toArray(new String[] {});
		}
		return null;
	}

	@Override
	public void enableColumnsUnfreeze(boolean enableUnfreeze, int... columns) {
		for (int column : columns) {
			enableColumnUnfreeze(enableUnfreeze, column);
		}
	}

	private void checkColumnsHideEnable() {
		// check column number
		for (int column = columnHideEnable.size() - 1; column >= 0; column--) {
			int col = Integer.parseInt(columnHideEnable.get(column)
					.substring(1));
			if (!(col >= 0 && col < allColumns)) {
				columnHideEnable.remove(column);
			}
		}
		// removed duplicated values
		if (!columnHideEnable.isEmpty()) {
			Set<String> tempColumns = new LinkedHashSet<String>(
					columnHideEnable);
			columnHideEnable = new ArrayList<String>(tempColumns);
		}
	}

	@Override
	public boolean canColumnHide(int column) {
		if (columnHideEnable.contains("-" + column)) {
			return false;
		} else if (columnHideEnable.contains("+" + column)) {
			return true;
		}
		return enableHideAnyColumns;
	}

	@Override
	public void enableColumnHide(boolean enableHide, int column) {
		try {
			columnHideEnable.remove("+" + column);
			columnHideEnable.remove("-" + column);
		} catch (Exception e) {
		}
		columnHideEnable.add((enableHide ? "+" : "-") + column);
	}

	@Override
	public String[] getColumnsHideEnabled(int[] columns) {
		if (enableHideAnyColumns || !columnHideEnable.isEmpty()) {
			checkColumnsHideEnable();
			List<Integer> cols = Arrays.asList(ArrayUtils.toObject(columns));
			List<String> tempCols = new ArrayList<String>();
			for (String column : columnHideEnable) {
				if (cols.contains(Integer.parseInt(column.substring(1)))) {
					tempCols.add(column);
				}
			}
			return tempCols.toArray(new String[] {});
		}
		return null;
	}

	@Override
	public String[] getColumnsHideEnabled() {
		if (enableHideAnyColumns && !columnHideEnable.isEmpty()) {
			checkColumnsHideEnable();
			return columnHideEnable.toArray(new String[] {});
		}
		return null;
	}

	@Override
	public void enableColumnsHide(boolean enableHide, int... columns) {
		for (int column : columns) {
			enableColumnHide(enableHide, column);
		}
	}

	private void checkColumnsUnhideEnable() {
		// check column number
		for (int column = columnUnhideEnable.size() - 1; column >= 0; column--) {
			int col = Integer.parseInt(columnUnhideEnable.get(column)
					.substring(1));
			if (!(col >= 0 && col < allColumns)) {
				columnUnhideEnable.remove(column);
			}
		}
		// removed duplicated values
		if (!columnUnhideEnable.isEmpty()) {
			Set<String> tempColumns = new LinkedHashSet<String>(
					columnUnhideEnable);
			columnUnhideEnable = new ArrayList<String>(tempColumns);
		}
	}

	@Override
	public boolean canColumnUnhide(int column) {
		if (columnUnhideEnable.contains("-" + column)) {
			return false;
		} else if (columnUnhideEnable.contains("+" + column)) {
			return true;
		}
		return enableUnhideAnyColumns;
	}

	@Override
	public void enableColumnUnhide(boolean enableUnhide, int column) {
		try {
			columnUnhideEnable.remove("+" + column);
			columnUnhideEnable.remove("-" + column);
		} catch (Exception e) {
		}
		columnUnhideEnable.add((enableUnhide ? "+" : "-") + column);
	}

	@Override
	public String[] getColumnsUnhideEnabled(int[] columns) {
		if (enableUnhideAnyColumns || !columnUnhideEnable.isEmpty()) {
			checkColumnsUnhideEnable();
			List<Integer> cols = Arrays.asList(ArrayUtils.toObject(columns));
			List<String> tempCols = new ArrayList<String>();
			for (String column : columnUnhideEnable) {
				if (cols.contains(Integer.parseInt(column.substring(1)))) {
					tempCols.add(column);
				}
			}
			return tempCols.toArray(new String[] {});
		}
		return null;
	}

	@Override
	public String[] getColumnsUnhideEnabled() {
		if (enableUnhideAnyColumns && !columnUnhideEnable.isEmpty()) {
			checkColumnsUnhideEnable();
			return columnUnhideEnable.toArray(new String[] {});
		}
		return null;
	}

	@Override
	public void enableColumnsUnhide(boolean enableUnhide, int... columns) {
		for (int column : columns) {
			enableColumnUnhide(enableUnhide, column);
		}
	}

	private void checkFormattingColumns() {
		if (!isNoFormattingColumn()) {
			for (int column = formattingColumns.size() - 1; column >= 0; column--) {
				if (!(formattingColumns.get(column).getColumn() >= 0 && formattingColumns
						.get(column).getColumn() < allColumns)) {
					formattingColumns.remove(column);
				}
			}
		}
	}

	private void replaceFormattingColumn(Formatting newFormattingColumns) {
		if (!isNoFormattingColumn()) {
			removeFormattingColumn(newFormattingColumns.getColumn());
		}
		formattingColumns.add(newFormattingColumns);
	}

	@Override
	public void removeFormattingColumn(int column) {
		if (!isNoFormattingColumn()) {
			if (isFormattingColumn(column)) {
				for (int col = 0; col < formattingColumns.size(); col++) {
					if (formattingColumns.get(col).equals(column)) {
						formattingColumns.remove(col);
						break;
					}
				}
			}
		}
	}

	@Override
	public boolean isNoFormattingColumn() {
		return !(formattingColumns != null && !formattingColumns.isEmpty());
	}

	@Override
	public boolean isFormattingColumn(int column) {
		if (!isNoFormattingColumn()) {
			for (Formatting formatting : formattingColumns) {
				if (formatting.getColumn().equals(column)) {
					return true;
				}
			}
		}
		return false;
	}

	@Override
	public Formatting getFormattingColumn(int column) {
		if (!isNoFormattingColumn()) {
			checkFormattingColumns();
			for (Formatting formatting : formattingColumns) {
				if (formatting.getColumn().equals(column)) {
					return formatting;
				}
			}
		}
		return null;
	}

	@Override
	public void setFormattingColumn(int column, Format format) {
		if (format != null) {
			replaceFormattingColumn(new Formatting(column, format));
		}
	}

	@Override
	public Formatting[] getFormattingColumnsArray() {
		if (!isNoFormattingColumn()) {
			checkFormattingColumns();
			return formattingColumns.toArray(new Formatting[] {});
		}
		return null;
	}

	@Override
	public int[] getFormattingColumns(int[] columns) {
		if (!isNoFormattingColumn() || columns == null || columns.length == 0) {
			checkFormattingColumns();
			List<Integer> cols = Arrays.asList(ArrayUtils.toObject(columns));
			List<Integer> tempCols = new ArrayList<Integer>();
			for (Formatting formatting : formattingColumns) {
				if (cols.contains(formatting.getColumn())) {
					tempCols.add(formatting.getColumn());
				}
			}
			return ArrayUtils.toPrimitive(tempCols.toArray(new Integer[] {}));
		}
		return null;
	}

	@Override
	public int[] getFormattingColumns() {
		if (!isNoFormattingColumn()) {
			List<Integer> tempCols = new ArrayList<Integer>();
			for (Formatting formatting : formattingColumns) {
				tempCols.add(formatting.getColumn());
			}
			return ArrayUtils.toPrimitive(tempCols.toArray(new Integer[] {}));
		}
		return null;
	}

	@Override
	public void setFormattingColumns(Format format, int... columns) {
		if (format != null && columns != null && columns.length > 0) {
			for (int column : columns) {
				setFormattingColumn(column, format);
			}
		}
	}

	public Map<String, Object> getPaintObjects() {
		Map<String, Object> paintObjects = new HashMap<String, Object>();

		paintObjects.put("pageResizingActionDelay",
				getPageResizingActionDelay());
		paintObjects.put("scrollbarActionDelay", getScrollbarActionDelay());
		paintObjects.put("actionTrayDelay", getActionTrayDelay());
		paintObjects.put("pageSize", getPageSize());
		paintObjects.put("pagerPages", getPagerPages());
		paintObjects.put("tableToolsPosition", getTableToolsPosition());
		paintObjects.put("filterInfoPosition", getFilterInfoPosition());
		paintObjects.put("tableFrozenMinimalWidth",
				getTableFrozenMinimalWidth());
		paintObjects.put("tableMinimalWidth", getTableMinimalWidth());
		paintObjects.put("contextMenuDepthForFilteredFrozenTable",
				getContextMenuDepthForFilteredFrozenTable());
		paintObjects.put("contextMenuDepthForFilteredTable",
				getContextMenuDepthForFilteredTable());
		paintObjects.put("tableHeight", getTableHeight());
		paintObjects.put("enableFixedTableHeight", isEnabledFixedTableHeight());

		paintObjects.put("headerRowHeight", header().getRowHeight());
		paintObjects.put("headerLineHeight", header().getLineHeight());
		paintObjects.put("headerCellMarginTop", header().getTopMargin());
		paintObjects.put("headerCellMarginRight", header().getRightMargin());
		paintObjects.put("headerCellMarginBottom", header().getBottomMargin());
		paintObjects.put("headerCellMarginLeft", header().getLeftMargin());
		paintObjects.put("headerRowsAlignment", header().getRowAlignment()
				.getAlignment());
		paintObjects.put("enableAllHeaderColumns", header()
				.isEnabledAllColumns());
		paintObjects.put("enableHeaderVerticalLines", header()
				.isEnabledVerticalLines());
		paintObjects.put("enableExpandedFrozenHeaderCell", header()
				.isEnabledExpandedFrozenCell());
		paintObjects.put("enableExpandedFrozenHeaderCellAlwaysVisible",
				header().isEnabledExpandedFrozenCellAlwaysVisible());
		paintObjects.put("enableExpandedFrozenHeaderCellForActiveColumn",
				header().isEnabledExpandedFrozenCellForActiveColumn());
		paintObjects.put("enableExpandedHeaderCell", header()
				.isEnabledExpandedCell());
		paintObjects.put("enableExpandedHeaderCellAlwaysVisible", header()
				.isEnabledExpandedCellAlwaysVisible());
		paintObjects.put("enableExpandedHeaderCellForActiveColumn", header()
				.isEnabledExpandedCellForActiveColumn());
		paintObjects.put("enableWordWrapInHeaderCells", header()
				.isEnabledWordWrap());
		paintObjects.put("enableTooltip", header().isEnabledTooltip());
		paintObjects.put("enableWordWrapInTooltip", header()
				.isEnabledWordWrapInTooltip());
		paintObjects.put("tooltipPosition", header().getTooltipPosition()
				.toString());
		paintObjects.put("tooltipMinimumWidth", header()
				.getTooltipMinimumWidth());
		paintObjects.put("tooltipMaximumWidth", header()
				.getTooltipMaximumWidth());

		paintObjects.put("bodyRowHeight", body().getRowHeight());
		paintObjects.put("bodyLineHeight", body().getLineHeight());
		paintObjects.put("bodyCellMarginTop", body().getTopMargin());
		paintObjects.put("bodyCellMarginRight", body().getRightMargin());
		paintObjects.put("bodyCellMarginBottom", body().getBottomMargin());
		paintObjects.put("bodyCellMarginLeft", body().getLeftMargin());
		paintObjects.put("bodyRowsAlignment", body().getRowAlignment()
				.getAlignment());
		paintObjects.put("enableBodyVerticalLines", body()
				.isEnabledVerticalLines());
		paintObjects.put("enableExpandedFrozenBodyCell", body()
				.isEnabledExpandedFrozenCell());
		paintObjects.put("enableExpandedFrozenBodyCellAlwaysVisible", body()
				.isEnabledExpandedFrozenCellAlwaysVisible());
		paintObjects.put("enableExpandedBodyCell", body()
				.isEnabledExpandedCell());
		paintObjects.put("enableExpandedBodyCellAlwaysVisible", body()
				.isEnabledExpandedCellAlwaysVisible());
		paintObjects.put("enableWordWrapInBodyCells", body()
				.isEnabledWordWrap());

		paintObjects.put("enableAllRows", isEnabledAllRows());
		paintObjects.put("enableAllColumns", isEnabledAllColumns());
		paintObjects.put("enableQuickSearch", isEnabledQuickSearch());
		paintObjects.put("enableQuickSearchImmediate",
				isEnabledQuickSearchImmediate());
		paintObjects.put("enableQuickSearchVisibleAtStartup",
				isEnabledQuickSearchVisibleAtStartup());
		paintObjects.put("enableQuickSearchHide", isEnabledQuickSearchHide());
		paintObjects.put("enableSearch", isEnabledSearch());
		paintObjects.put("enableSearchFromatting", isEnabledSearchFromatting());
		paintObjects.put("enableSearchImmediate", isEnabledSearchImmediate());
		paintObjects.put("enableSearchVisibleAtStartup",
				isEnabledSearchVisibleAtStartup());
		paintObjects.put("enableSearchHide", isEnabledSearchHide());
		paintObjects.put("enableColumnNumberInWaitBox",
				isEnabledColumnNumberInLoadingMessageBox());
		paintObjects.put("enableFrozenTable", isEnabledFrozenTable());
		paintObjects.put("enableFrozenTableVisible",
				isEnabledFrozenTableVisible());
		paintObjects.put("enableHideFrozenTable", isEnabledHideFrozenTable());
		paintObjects.put("enableFilteredFrozenTable",
				isEnabledFilteredFrozenTable());
		paintObjects.put("enableFilteredTable", isEnabledFilteredTable());
		paintObjects.put("enableContextMenuForFilteredFrozenTable",
				isEnabledContextMenuForFilteredFrozenTable());
		paintObjects.put("enableContextMenuForFilteredTable",
				isEnabledContextMenuForFilteredTable());
		paintObjects.put("enableEmptyFrozenTable", isEnabledEmptyFrozenTable());
		paintObjects.put("enableResizeFrozenTable",
				isEnabledResizeFrozenTable());
		paintObjects.put("enableOrderedFrozenTable",
				isEnabledOrderedFrozenTable());
		paintObjects.put("enableOrderedTable", isEnabledOrderedTable());
		paintObjects.put("enableFilterInfo", isEnabledFilterInfo());
		paintObjects.put("enablePageableTable", isEnabledPageableTable());
		paintObjects.put("enableFreezeUnfreezeColumns",
				isEnabledFreezeUnfreezeColumns());
		paintObjects.put("enableFreezeAnyColumns", isEnabledFreezeAnyColumns());
		paintObjects.put("enableUnfreezeAnyColumns",
				isEnabledUnfreezeAnyColumns());
		paintObjects.put("enableHideUnhideColumns",
				isEnabledHideUnhideColumns());
		paintObjects.put("enableHideAnyColumns", isEnabledHideAnyColumns());
		paintObjects.put("enableHorizontalLines", isEnabledHorizontalLines());
		paintObjects.put("enableHighlightActiveRow",
				isEnabledHighlightActiveRow());
		paintObjects.put("enableOddAndEvenColoringRows",
				isEnabledOddAndEvenColoringRows());
		paintObjects.put("enableLoadingMessage", isEnabledLoadingMessage());
		paintObjects.put("loadingMessageOpacity", ""
				+ getLoadingMessageOpacity());
		paintObjects.put("loadingMessageShowDelay",
				getLoadingMessageShowDelay());
		paintObjects.put("loadingMessageHideDelay",
				getLoadingMessageHideDelay());

		return paintObjects;
	}

	public Map<String, Object> getPaintObjects(int[] columns,
			int[] headerColumns) {
		Map<String, Object> paintObjects = new HashMap<String, Object>();

		paintObjects.put("headerColumnsAlignment", header()
				.getColumnsAlignmentIntArray(headerColumns));
		paintObjects.put("headerColumnsWidth", getColumnsWidth(headerColumns));
		paintObjects.put("bodyColumnsAlignment", body()
				.getColumnsAlignmentIntArray(columns));
		paintObjects.put("columnsWidth", getColumnsWidth(columns));
		paintObjects.put("formatedColumns", getFormattingColumns(columns));
		paintObjects.put("enableFreezeColumns",
				getColumnsFreezeEnabled(columns));
		paintObjects.put("enableUnfreezeColumns",
				getColumnsUnfreezeEnabled(columns));
		paintObjects.put("enableHideColumns", getColumnsHideEnabled(columns));

		return paintObjects;
	}

}
