/**
 * Copyright (C) 2012  Dušan Vejnovič  <vaadin@dussan.org>
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 * http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */
package org.dussan.vaadin.dtable.config.tablesection;

import java.util.ArrayList;
import java.util.Arrays;
import java.util.Collections;
import java.util.HashMap;
import java.util.List;
import java.util.Map;

import org.apache.commons.lang3.ArrayUtils;
import org.dussan.vaadin.dtable.config.tablesection.alignments.ColumnAlign;
import org.dussan.vaadin.dtable.config.tablesection.alignments.ColumnAlignment;
import org.dussan.vaadin.dtable.config.tablesection.alignments.RowAlign;
import org.dussan.vaadin.dtable.config.tablesection.alignments.RowAlignment;
import org.dussan.vaadin.dtable.config.tablesection.dimensions.RowHeight;
import org.dussan.vaadin.dtable.config.tablesection.margins.Margin;

public class TableSectionConfig implements RowHeight, ColumnAlignment,
		RowAlignment, Margin {

	private boolean enableVerticalLines = false;
	private boolean enableExpandedFrozenCell = false;
	private boolean enableExpandedFrozenCellAlwaysVisible = false;
	private boolean enableExpandedCell = false;
	private boolean enableExpandedCellAlwaysVisible = false;
	private boolean enableWordWrap = false;

	private boolean checkColumnsAlignment = true;
	private int allColumns = 0;
	private int rowHeight = RowHeight.ROW_DEFAULT_HEIGHT;
	private int lineHeight = RowHeight.LINE_DEFAULT_HEIGHT;
	private ColumnAlign columnDefaultAlignment = ColumnAlign.LEFT;
	private Map<Integer, ColumnAlign> columnsAlignment = new HashMap<Integer, ColumnAlign>();
	private RowAlign rowAlignment = RowAlign.MIDDLE;
	private int topMargin = 0;
	private int rightMargin = 0;
	private int bottomMargin = 0;
	private int leftMargin = 0;

	public void setAllColumns(int allColumns) {
		this.allColumns = allColumns;
	}

	private boolean isHeightOK(int height) {
		return (height > 0);
	}

	public boolean isEnabledVerticalLines() {
		return enableVerticalLines;
	}

	public void enableVerticalLines(boolean enableVerticalLines) {
		this.enableVerticalLines = enableVerticalLines;
	}

	public boolean isEnabledExpandedFrozenCell() {
		return enableExpandedFrozenCell;
	}

	public void enableExpandedFrozenCell(boolean enableExpandedFrozenCell) {
		this.enableExpandedFrozenCell = enableExpandedFrozenCell;
	}

	public void enableExpandedFrozenCell(boolean enableExpandedFrozenCell,
			boolean enableExpandedFrozenCellAlwaysVisible) {
		this.enableExpandedFrozenCell = enableExpandedFrozenCell;
		this.enableExpandedFrozenCellAlwaysVisible = enableExpandedFrozenCellAlwaysVisible;
	}

	public boolean isEnabledExpandedFrozenCellAlwaysVisible() {
		return enableExpandedFrozenCellAlwaysVisible;
	}

	public void enableExpandedFrozenCellAlwaysVisible(
			boolean enableExpandedFrozenCellAlwaysVisible) {
		this.enableExpandedFrozenCellAlwaysVisible = enableExpandedFrozenCellAlwaysVisible;
	}

	public boolean isEnabledExpandedCell() {
		return enableExpandedCell;
	}

	public void enableExpandedCell(boolean enableExpandedCell) {
		this.enableExpandedCell = enableExpandedCell;
	}

	public void enableExpandedCell(boolean enableExpandedCell,
			boolean enableExpandedCellAlwaysVisible) {
		this.enableExpandedCell = enableExpandedCell;
		this.enableExpandedCellAlwaysVisible = enableExpandedCellAlwaysVisible;
	}

	public boolean isEnabledExpandedCellAlwaysVisible() {
		return enableExpandedCellAlwaysVisible;
	}

	public void enableExpandedCellAlwaysVisible(
			boolean enableExpandedCellAlwaysVisible) {
		this.enableExpandedCellAlwaysVisible = enableExpandedCellAlwaysVisible;
	}

	public boolean isEnabledWordWrap() {
		return enableWordWrap;
	}

	public void enableWordWrap(boolean enableWordWrap) {
		this.enableWordWrap = enableWordWrap;
	}

	@Override
	public int getRowHeight() {
		return rowHeight;
	}

	@Override
	public void setRowHeight(int height) {
		rowHeight = isHeightOK(height) ? height : rowHeight;
	}

	@Override
	public int getLineHeight() {
		return lineHeight;
	}

	@Override
	public void setLineHeight(int height) {
		lineHeight = isHeightOK(height) ? height : lineHeight;
	}

	@Override
	public ColumnAlign getDefaultColumnAlignment() {
		return columnDefaultAlignment;
	}

	@Override
	public void setDefaultColumnAlignment(ColumnAlign alignment) {
		columnDefaultAlignment = alignment != null ? alignment
				: columnDefaultAlignment;
	}

	private void checkColumnsAlignment() {
		if (checkColumnsAlignment && allColumns > 0) {
			if (!columnsAlignment.isEmpty()) {
				// check columns number
				List<Integer> columns = Arrays.asList(columnsAlignment.keySet()
						.toArray(new Integer[] {}));
				Collections.sort(columns);
				if (!columns.isEmpty()
						&& !(columns.get(0) >= 0 && columns
								.get(columns.size() - 1) < allColumns)) {
					Map<Integer, ColumnAlign> alignments = new HashMap<Integer, ColumnAlign>();
					for (int column : columns) {
						if (column >= 0 && column < allColumns) {
							alignments
									.put(column, columnsAlignment.get(column));
						}
					}
					columnsAlignment = new HashMap<Integer, ColumnAlign>(
							alignments);
				}
			}

			// add missing columns width
			if (columnsAlignment.size() < allColumns) {
				for (int column = 0; column < allColumns; column++) {
					if (!columnsAlignment.containsKey(column)) {
						columnsAlignment.put(column, columnDefaultAlignment);
					}
				}
			}
			checkColumnsAlignment = false;
		}
	}

	@Override
	public ColumnAlign getColumnAlignment(int column) {
		try {
			columnsAlignment.get(column);
		} catch (Exception e) {
		}
		return columnDefaultAlignment;
	}

	@Override
	public void setColumnAlignment(int column, ColumnAlign alignment) {
		checkColumnsAlignment = true;
		columnsAlignment.put(column, alignment != null ? alignment
				: columnDefaultAlignment);
	}

	@Override
	public ColumnAlign[] getColumnsAlignment() {
		checkColumnsAlignment();
		ColumnAlign[] alignments = new ColumnAlign[columnsAlignment.size()];
		for (int column = 0; column < columnsAlignment.size(); column++) {
			alignments[column] = columnsAlignment.get(column);
		}
		return alignments;
	}

	@Override
	public int[] getColumnsAlignmentIntArray() {
		checkColumnsAlignment();
		int[] alignments = new int[columnsAlignment.size()];
		for (int column = 0; column < columnsAlignment.size(); column++) {
			alignments[column] = columnsAlignment.get(column).getAlignment();
		}
		return alignments;
	}

	@Override
	public int[] getColumnsAlignmentIntArray(int[] columns) {
		checkColumnsAlignment();
		List<Integer> alignments = new ArrayList<Integer>();
		for (int column : columns) {
			alignments.add(columnsAlignment.get(column).getAlignment());
		}
		return ArrayUtils.toPrimitive(alignments.toArray(new Integer[] {}));
	}

	@Override
	public void setColumnsAlignment(ColumnAlign... alignments) {
		checkColumnsAlignment = true;
		for (int column = 0; column < alignments.length; column++) {
			setColumnAlignment(column, alignments[column]);
		}
	}

	@Override
	public void setColumnsAlignment(ColumnAlign alignment, int... columns) {
		checkColumnsAlignment = true;
		for (int column : columns) {
			setColumnAlignment(column, alignment);
		}
	}

	@Override
	public void setColumnsAlignment(Map<Integer, ColumnAlign> alignments) {
		checkColumnsAlignment = true;
		for (int column : alignments.keySet()) {
			setColumnAlignment(column, alignments.get(column));
		}
	}

	@Override
	public RowAlign getRowAlignment() {
		return rowAlignment;
	}

	@Override
	public void setRowAlignment(RowAlign alignment) {
		rowAlignment = alignment != null ? alignment : rowAlignment;
	}

	@Override
	public int getTopMargin() {
		return topMargin;
	}

	@Override
	public int getRightMargin() {
		return rightMargin;
	}

	@Override
	public int getBottomMargin() {
		return bottomMargin;
	}

	@Override
	public int getLeftMargin() {
		return leftMargin;
	}

	@Override
	public void setMargins(int topMargin, int rightMargin, int bottomMargin,
			int leftMargin) {
		this.topMargin = topMargin > 0 ? topMargin : this.topMargin;
		this.rightMargin = rightMargin > 0 ? rightMargin : this.rightMargin;
		this.bottomMargin = bottomMargin > 0 ? bottomMargin : this.bottomMargin;
		this.leftMargin = leftMargin > 0 ? leftMargin : this.leftMargin;
	};

	@Override
	public void setMargins(int topBottomMargin, int leftRightMargin) {
		this.topMargin = topBottomMargin > 0 ? topBottomMargin : this.topMargin;
		this.rightMargin = leftRightMargin > 0 ? leftRightMargin
				: this.rightMargin;
		this.bottomMargin = topBottomMargin > 0 ? topBottomMargin
				: this.bottomMargin;
		this.leftMargin = leftRightMargin > 0 ? leftRightMargin
				: this.leftMargin;
	};

	@Override
	public void setTopBottomMargins(int topMargin, int bottomMargin) {
		this.topMargin = topMargin > 0 ? topMargin : this.topMargin;
		this.bottomMargin = bottomMargin > 0 ? bottomMargin : this.bottomMargin;
	};

	@Override
	public void setTopBottomMargins(int margin) {
		topMargin = margin > 0 ? margin : topMargin;
		bottomMargin = margin > 0 ? margin : bottomMargin;
	};

	@Override
	public void setLeftRightMargins(int leftMargin, int rightMargin) {
		this.leftMargin = leftMargin > 0 ? leftMargin : this.leftMargin;
		this.rightMargin = rightMargin > 0 ? rightMargin : this.rightMargin;
	};

	@Override
	public void setLeftRightMargins(int margin) {
		leftMargin = margin > 0 ? margin : leftMargin;
		rightMargin = margin > 0 ? margin : rightMargin;
	};

}
