/**
 * Copyright (C) 2012  Dušan Vejnovič  <vaadin@dussan.org>
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 * http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */
package org.dussan.vaadin.dtable.query;

import java.io.Serializable;
import java.util.ArrayList;
import java.util.Arrays;
import java.util.HashMap;
import java.util.LinkedHashSet;
import java.util.List;
import java.util.Map;
import java.util.Set;

import org.dussan.vaadin.dtable.query.elements.Field;
import org.dussan.vaadin.dtable.query.elements.Functions;
import org.dussan.vaadin.dtable.query.elements.Having;
import org.dussan.vaadin.dtable.query.elements.HavingType;
import org.dussan.vaadin.dtable.query.elements.OrderBy;
import org.dussan.vaadin.dtable.query.elements.TablesJoin;
import org.dussan.vaadin.dtable.query.elements.Where;
import org.dussan.vaadin.dtable.query.elements.WhereType;

public class Query implements Serializable {

	private static final long serialVersionUID = -4764808594557072651L;
	private long startRow = 0;
	private int rows = 0;
	private String fromTable = null;
	private String fromTableAlias = null;
	private Map<Integer, Field> fields = null;
	private List<TablesJoin> tableJoins = null;
	private List<Object> whereCondition = null;
	private WhereType whereMainType = WhereType.AND;
	private List<Object> groupBy = null;
	private List<Object> havingCondition = null;
	private HavingType havingMainType = HavingType.OR;
	private List<OrderBy> orderBy = null;

	public Query() {
		fromTable = new String();
		fromTableAlias = new String();
		fields = new HashMap<Integer, Field>();
		tableJoins = new ArrayList<TablesJoin>();
		whereCondition = new ArrayList<Object>();
		groupBy = new ArrayList<Object>();
		havingCondition = new ArrayList<Object>();
		orderBy = new ArrayList<OrderBy>();
	}

	private Query(long startRow, int rows, String fromTable,
			String fromTableAlias, Map<Integer, Field> fields,
			List<TablesJoin> tableJoins, List<Object> whereCondition,
			WhereType whereMainMerge, List<Object> groupBy,
			List<Object> havingCondition, HavingType havingMainMerge,
			List<OrderBy> orderBy) {
		this.startRow = new Long(startRow);
		this.rows = new Integer(rows);
		this.fromTable = fromTable != null ? new String(fromTable)
				: new String();
		this.fromTableAlias = fromTableAlias != null ? new String(
				fromTableAlias) : new String();
		this.fields = fields != null ? new HashMap<Integer, Field>(fields)
				: new HashMap<Integer, Field>();
		this.tableJoins = tableJoins != null ? new ArrayList<TablesJoin>(
				tableJoins) : new ArrayList<TablesJoin>();
		this.whereCondition = whereCondition != null ? new ArrayList<Object>(
				whereCondition) : new ArrayList<Object>();
		this.whereMainType = whereMainMerge;
		this.groupBy = groupBy != null ? new ArrayList<Object>(groupBy)
				: new ArrayList<Object>();
		this.havingCondition = havingCondition != null ? new ArrayList<Object>(
				havingCondition) : new ArrayList<Object>();
		this.havingMainType = havingMainMerge;
		this.orderBy = orderBy != null ? new ArrayList<OrderBy>(orderBy)
				: new ArrayList<OrderBy>();
	}

	public static Query clone(Query query) {
		return new Query(query.startRow, query.rows, query.fromTable,
				query.fromTableAlias, query.fields, query.tableJoins,
				query.whereCondition, query.whereMainType, query.groupBy,
				query.havingCondition, query.havingMainType, query.orderBy);
	}

	public Query addLimit(long startRow, int rows) {
		this.startRow = startRow;
		this.rows = rows;
		return this;
	}

	public Query addFromTable(String fromTable, String fromTableAlias) {
		this.fromTable = fromTable;
		this.fromTableAlias = fromTableAlias;
		return this;
	}

	public Query addFromTable(String fromTable) {
		this.fromTable = fromTable;
		this.fromTableAlias = fromTable;
		return this;
	}

	public void addFromTableAlias(String fromTableAlias) {
		this.fromTableAlias = fromTableAlias;
	}

	public Map<Integer, Field> getFields() {
		return fields;
	}

	public Query resetFields() {
		fields.clear();
		return this;
	}

	public void removeFields(int... columns) {
		if (columns != null && columns.length > 0) {
			for (int column : columns) {
				fields.remove(column);
			}
		}
	}

	public void removeFields(String... fields) {
		if (fields != null && fields.length > 0) {
			for (String field : fields) {
				for (int column : this.fields.keySet()) {
					if (this.fields.get(column).getName().equals(field)) {
						this.fields.remove(column);
						break;
					}
				}
			}
		}
	}

	public Query addField(int column, Field field) {
		fields.put(column, field);
		return this;
	}

	public Query addFields(Map<Integer, Field> fields) {
		this.fields.putAll(fields);
		return this;
	}

	public List<TablesJoin> getTableJoins() {
		return tableJoins;
	}

	public void resetJoins() {
		tableJoins.clear();
	}

	public Query addJoins(TablesJoin... joins) {
		this.tableJoins.addAll(Arrays.asList(joins));
		Set<TablesJoin> withoutDuplicates = new LinkedHashSet<TablesJoin>(
				this.tableJoins);
		this.tableJoins = new ArrayList<TablesJoin>(withoutDuplicates);
		return this;
	}

	public Query addJoin(TablesJoin join) {
		addJoins(join);
		return this;
	}

	private String getSqlWhereCondition() {
		if (whereCondition != null && !whereCondition.isEmpty()) {
			switch (whereMainType) {
			case AND:
				return Where.AND(whereCondition.toArray());
			case OR:
				return Where.OR(whereCondition.toArray());
			}
		}
		return null;
	}

	public List<Object> getWhereCondition() {
		return whereCondition;
	}

	public WhereType getWhereMainType() {
		return whereMainType;
	}

	public Query setWhereMainType(WhereType whereMainType) {
		this.whereMainType = whereMainType;
		return this;
	}

	public void resetWhereCondition() {
		whereCondition.clear();
	}

	public Query addWhereCondition(Object... whereCondition) {
		this.whereCondition.addAll(Arrays.asList(whereCondition));
		Set<Object> withoutDuplicates = new LinkedHashSet<Object>(
				this.whereCondition);
		this.whereCondition = new ArrayList<Object>(withoutDuplicates);
		return this;
	}

	public Query addWhereCondition(Where whereCondition) {
		addWhereCondition(whereCondition);
		return this;
	}

	public List<Object> getGroupBy() {
		return groupBy;
	}

	public void resetGroupBy() {
		groupBy.clear();
	}

	public void removeGroupBy(String... fields) {
		if (fields != null && fields.length > 0) {
			for (String field : fields) {
				for (Object group : this.groupBy) {
					if (group.equals(field)) {
						this.groupBy.remove(this.groupBy.indexOf(group));
						break;
					}
				}
			}
		}
	}

	public Query addGroupBy(Object... groupBy) {
		this.groupBy.addAll(Arrays.asList(groupBy));
		Set<Object> withoutDuplicates = new LinkedHashSet<Object>(this.groupBy);
		this.groupBy = new ArrayList<Object>(withoutDuplicates);

		for (int column : fields.keySet()) {
			if (this.groupBy.contains(fields.get(column).getAlias())
					&& !Functions.NONE.equals(fields.get(column).getFunction())) {
				removeGroupBy(fields.get(column).getAlias());
			}
		}

		return this;
	}

	public List<OrderBy> getOrderBy() {
		return orderBy;
	}

	public void resetOrderBy() {
		orderBy.clear();
	}

	public void removeOrderBy(String... fields) {
		if (fields != null && fields.length > 0) {
			for (String field : fields) {
				for (OrderBy order : this.orderBy) {
					if (order.getField().equals(field)) {
						this.orderBy.remove(this.orderBy.indexOf(order));
						break;
					}
				}
			}
		}
	}

	public void removeOrderBy(OrderBy... orders) {
		if (orders != null && orders.length > 0) {
			for (OrderBy order : orders) {
				removeOrderBy(order.getField());
			}
		}
	}

	public Query addOrderBy(OrderBy... orderBy) {
		removeOrderBy(orderBy);
		this.orderBy.addAll(Arrays.asList(orderBy));
		Set<OrderBy> withoutDuplicates = new LinkedHashSet<OrderBy>(
				this.orderBy);
		this.orderBy = new ArrayList<OrderBy>(withoutDuplicates);
		return this;
	}

	private String getSqlHavingCondition() {
		if (havingCondition != null && !havingCondition.isEmpty()) {
			switch (whereMainType) {
			case AND:
				return Having.AND(havingCondition.toArray());
			case OR:
				return Having.OR(havingCondition.toArray());
			}
		}
		return null;
	}

	public List<Object> getHavingCondition() {
		return havingCondition;
	}

	public HavingType getHavingMainType() {
		return havingMainType;
	}

	public Query setHavingMainType(HavingType havingMainType) {
		this.havingMainType = havingMainType;
		return this;
	}

	public void resetHavingCondition() {
		havingCondition.clear();
	}

	public Query addHavingCondition(Object... havingCondition) {
		this.havingCondition.addAll(Arrays.asList(havingCondition));
		Set<Object> withoutDuplicates = new LinkedHashSet<Object>(
				this.havingCondition);
		this.havingCondition = new ArrayList<Object>(withoutDuplicates);
		return this;
	}

	public Query addHavingCondition(Having havingCondition) {
		addHavingCondition(havingCondition);
		return this;
	}

	private String getSqlSelect() {
		if (fields != null && !fields.isEmpty()) {
			StringBuilder selectBuilder = new StringBuilder();
			for (int column : fields.keySet()) {
				selectBuilder.append(",");
				selectBuilder.append(fields.get(column).getSqlField());
			}
			return ("SELECT " + selectBuilder.substring(",".length()));
		}
		return null;
	}

	private String getSqlSelectFrom() {
		if (!fields.isEmpty() && fromTable != null && !fromTable.isEmpty()) {
			return (getSqlSelect() + " FROM " + fromTable + " " + fromTableAlias);
		}
		return null;
	}

	private String getSqlSelectFromWithJoins() {
		if (getSqlSelectFrom() != null && tableJoins != null
				&& !tableJoins.isEmpty()) {
			StringBuilder joinsBuilder = new StringBuilder();
			for (TablesJoin join : tableJoins) {
				joinsBuilder.append(" ");
				joinsBuilder.append(join.getTablesJoin());
			}
			return (getSqlSelectFrom() + " " + joinsBuilder.substring(" "
					.length()));
		}
		return getSqlSelectFrom();
	}

	private String getSqlSelectFromWithWhereCondition() {
		if (getSqlWhereCondition() != null && !getSqlWhereCondition().isEmpty()) {
			return (getSqlSelectFromWithJoins() + " WHERE " + getSqlWhereCondition());
		}
		return getSqlSelectFromWithJoins();
	}

	private String getSqlSelectFromWithWhereConditionAndGroupBy() {
		if (groupBy != null && !groupBy.isEmpty()) {
			StringBuilder groupingBuilder = new StringBuilder();
			for (Object groupBy : this.groupBy) {
				groupingBuilder.append(",");
				groupingBuilder.append(groupBy);
			}
			return (getSqlSelectFromWithWhereCondition() + " GROUP BY " + groupingBuilder
					.substring(",".length()));
		}
		return getSqlSelectFromWithWhereCondition();
	}

	private String getSqlSelectFromWithWhereConditionGroupByAndHavingCondition() {
		if (havingCondition != null && !havingCondition.isEmpty()) {
			return (getSqlSelectFromWithWhereConditionAndGroupBy() + " HAVING " + getSqlHavingCondition());
		}
		return getSqlSelectFromWithWhereConditionAndGroupBy();
	}

	private String getSqlSelectFromWithWhereConditionGroupByHavingConditionAndOrderBy() {
		if (orderBy != null && !orderBy.isEmpty()) {
			StringBuilder orderBuilder = new StringBuilder();
			for (OrderBy orderBy : this.orderBy) {
				orderBuilder.append(",");
				orderBuilder.append(orderBy.getField() + " "
						+ orderBy.getOrderBy());
			}
			return (getSqlSelectFromWithWhereConditionGroupByAndHavingCondition()
					+ " ORDER BY " + orderBuilder.substring(",".length()));
		}
		return getSqlSelectFromWithWhereConditionGroupByAndHavingCondition();
	}

	public String getAllRowsCounterQueryString() {
		return ("SELECT count(*) FROM ("
				+ getSqlSelectFromWithWhereConditionGroupByAndHavingCondition() + ") AS allRows");
	}

	public String getQueryString() {
		if (startRow >= 0 && rows > 0) {
			return (getSqlSelectFromWithWhereConditionGroupByHavingConditionAndOrderBy()
					+ " LIMIT " + startRow + "," + rows);
		}
		return getSqlSelectFromWithWhereConditionGroupByHavingConditionAndOrderBy();
	}

}
