/**
 * Copyright (C) 2012  Dušan Vejnovič  <vaadin@dussan.org>
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 * http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */
package org.dussan.vaadin.dtable.windows.exporter;

import java.io.Serializable;
import java.util.ArrayList;
import java.util.Arrays;
import java.util.HashMap;
import java.util.List;
import java.util.Map;

import org.dussan.vaadin.dtable.config.formatting.Formatting;
import org.dussan.vaadin.dtable.windows.exporter.base.Charset;
import org.dussan.vaadin.dtable.windows.exporter.base.CompressionType;
import org.dussan.vaadin.dtable.windows.exporter.base.ExportingColumnType;
import org.dussan.vaadin.dtable.windows.exporter.base.MsFileVersion;

public class TableExporter implements Serializable {

	private static final long serialVersionUID = -6777208258494068964L;

	private Map<String, Map<Integer, Object[]>> columnsDefinition = new HashMap<String, Map<Integer, Object[]>>();

	private boolean enableExportCsvFile = false;
	private boolean enableExportMdbFile = false;
	private boolean enableExportPdfFile = false;
	private boolean enableExportXlsFile = false;

	private boolean enableFormattedData = false;
	private boolean enableTimestamp = false;
	private String timestampFormat = "yyyyMMdd-HHmmss";

	private String filename = "dtable";
	private String basename = "dtable";
	private String csvColumnDelimiter = ";";

	private MsFileVersion msFileVersion = MsFileVersion.V2003;
	private Charset charset = Charset.UTF_8;
	private CompressionType compressionType = CompressionType.NONE;
	private Map<String, Long> allRows = new HashMap<String, Long>();

	public TableExporter() {
		createColumnDefinition(basename);
	}

	public static Object[] arrayList2objectArray(List<Object[]> datas) {
		try {
			List<Object> objects = new ArrayList<Object>();
			for (Object[] data : datas) {
				objects.addAll(Arrays.asList(data));
			}
			return objects.toArray();
		} catch (Exception e) {
			return null;
		}
	}

	public String getBasename() {
		return basename;
	}

	public void setBasename(String basename) {
		if (basename != null && !basename.isEmpty()) {
			Map<Integer, Object[]> definition = getColumnDefinition(getBasename());
			removeColumnDefinition(getBasename());
			columnsDefinition.put(this.basename = basename, definition);
		}
	}

	public long getAllRows(String name) {
		if (allRows.containsKey(name)) {
			return allRows.get(name);
		}
		return 0L;
	}

	public void setAllRows(String name, Long allRows) {
		this.allRows.put(name, allRows);
	}

	public void checkBaseColumnsDefinition(String[] columnsName,
			Formatting[] formattings, int[] removedColumns) {
		Map<Integer, Object[]> definition = getColumnDefinition(getBasename());
		for (int column = columnsName.length - 1; column >= 0; column--) {
			if (!definition.containsKey(column)) {
				Formatting format = null;
				if (formattings != null && formattings.length > 0) {
					for (Formatting formatting : formattings) {
						if (formatting.getColumn() == column) {
							format = formatting;
							break;
						}
					}
				}
				definition.put(column, new Object[] { columnsName[column],
						format, null, null });
			}
			for (int col : removedColumns) {
				if (col == column) {
					definition.remove(Integer.parseInt("" + col));
				}
			}
		}
		columnsDefinition.put(getBasename(), definition);
	}

	public String[] getColumnsDefinitionName() {
		return columnsDefinition.keySet().toArray(new String[] {});
	}

	public void createColumnDefinition(String name) {
		columnsDefinition.put(name, new HashMap<Integer, Object[]>());
	}

	public boolean isColumnDefinitionEmpty(String name) {
		if (!columnsDefinition.containsKey(name)) {
			return true;
		}
		return (columnsDefinition.get(name).isEmpty());
	}

	public void removeColumnDefinition(String name) {
		columnsDefinition.remove(name);
	}

	public Map<String, Map<Integer, Object[]>> getColumnsDefinition() {
		return columnsDefinition;
	}

	public Map<Integer, Object[]> getColumnDefinition(String name) {
		return columnsDefinition.get(name);
	}

	public String getColumnDefinitionHeader(String name, int column) {
		Map<Integer, Object[]> columnsDefinition = getColumnDefinition(name);
		if (columnsDefinition != null && columnsDefinition.containsKey(column)) {
			return getColumnDefinition(name).get(column)[0].toString();
		}
		return null;
	}

	public Formatting getColumnDefinitionFormatting(String name, int column) {
		Map<Integer, Object[]> columnsDefinition = getColumnDefinition(name);
		if (columnsDefinition != null && columnsDefinition.containsKey(column)) {
			return (Formatting) getColumnDefinition(name).get(column)[1];
		}
		return null;
	}

	public ExportingColumnType getColumnDefinitionExportingType(String name,
			int column) {
		Map<Integer, Object[]> columnsDefinition = getColumnDefinition(name);
		if (columnsDefinition != null && columnsDefinition.containsKey(column)) {
			return (ExportingColumnType) getColumnDefinition(name).get(column)[2];
		}
		return null;
	}

	public Integer getColumnDefinitionLengthOrPrecision(String name, int column) {
		Map<Integer, Object[]> columnsDefinition = getColumnDefinition(name);
		if (columnsDefinition != null && columnsDefinition.containsKey(column)) {
			return (Integer) getColumnDefinition(name).get(column)[3];
		}
		return null;
	}

	public void setColumnDefinition(String name,
			Map<Integer, Object[]> definition) {
		columnsDefinition.put(name, definition);
	}

	public void setColumnDefinition(String name, int column, String columnHeader) {
		if (columnsDefinition.containsKey(name)) {
			Map<Integer, Object[]> exported = new HashMap<Integer, Object[]>();
			if (!isColumnDefinitionEmpty(name)) {
				exported = columnsDefinition.get(name);
			}
			exported.put(column,
					new Object[] { columnHeader, null, null, null });
			columnsDefinition.put(name, exported);
		}
	}

	public void setColumnDefinition(String name, int column,
			String columnHeader, Formatting formatting) {
		if (formatting != null) {
			formatting.setColumn(column);
		}
		if (columnsDefinition.containsKey(name)) {
			Map<Integer, Object[]> exported = new HashMap<Integer, Object[]>();
			if (!isColumnDefinitionEmpty(name)) {
				exported = columnsDefinition.get(name);
			}
			exported.put(column, new Object[] { columnHeader, formatting, null,
					null });
			columnsDefinition.put(name, exported);
		}
	}

	public void setColumnDefinition(String name, int column,
			String columnHeader, Formatting formatting,
			ExportingColumnType exportedColumnType) {
		if (formatting != null) {
			formatting.setColumn(column);
		}
		if (columnsDefinition.containsKey(name)) {
			Map<Integer, Object[]> exported = new HashMap<Integer, Object[]>();
			if (!isColumnDefinitionEmpty(name)) {
				exported = columnsDefinition.get(name);
			}
			exported.put(column, new Object[] { columnHeader, formatting,
					exportedColumnType, null });
			columnsDefinition.put(name, exported);
		}
	}

	public void setColumnDefinition(String name, int column,
			String columnHeader, Formatting formatting,
			ExportingColumnType exportedColumnType, int lengthOrPrecision) {
		if (formatting != null) {
			formatting.setColumn(column);
		}
		if (columnsDefinition.containsKey(name)) {
			Map<Integer, Object[]> exported = new HashMap<Integer, Object[]>();
			if (!isColumnDefinitionEmpty(name)) {
				exported = columnsDefinition.get(name);
			}
			exported.put(column, new Object[] { columnHeader, formatting,
					exportedColumnType, lengthOrPrecision });
			columnsDefinition.put(name, exported);
		}
	}

	public boolean isEnabledExportCsvFile() {
		return enableExportCsvFile;
	}

	public void enableExportCsvFile(boolean enableExportCsvFile) {
		this.enableExportCsvFile = enableExportCsvFile;
	}

	public boolean isEnabledExportMdbFile() {
		return enableExportMdbFile;
	}

	public void enableExportMdbFile(boolean enableExportMdbFile) {
		this.enableExportMdbFile = enableExportMdbFile;
	}

	public boolean isEnabledExportPdfFile() {
		return enableExportPdfFile;
	}

	public void enableExportPdfFile(boolean enableExportPdfFile) {
		this.enableExportPdfFile = enableExportPdfFile;
	}

	public boolean isEnabledExportXlsFile() {
		return enableExportXlsFile;
	}

	public void enableExportXlsFile(boolean enableExportXlsFile) {
		this.enableExportXlsFile = enableExportXlsFile;
	}

	public Map<String, Object> getPaintObjects() {
		Map<String, Object> paintObjects = new HashMap<String, Object>();
		paintObjects.put("enableExportCsvFile", isEnabledExportCsvFile());
		paintObjects.put("enableExportMdbFile", isEnabledExportMdbFile());
		paintObjects.put("enableExportPdfFile", isEnabledExportPdfFile());
		paintObjects.put("enableExportXlsFile", isEnabledExportXlsFile());
		return paintObjects;
	}

	public boolean isEnabledFormattedData() {
		return enableFormattedData;
	}

	public void enableFormattedData(boolean enableFormattedData) {
		this.enableFormattedData = enableFormattedData;
	}

	public boolean isEnabledTimestamp() {
		return enableTimestamp;
	}

	public void enableTimestamp(boolean enableTimestamp) {
		this.enableTimestamp = enableTimestamp;
	}

	public String getTimestampFormat() {
		return timestampFormat;
	}

	public void setTimestampFormat(String timestampFormat) {
		this.timestampFormat = timestampFormat;
	}

	public String getFilename() {
		return filename;
	}

	public void setFilename(String filename) {
		this.filename = filename;
	}

	public String getCsvColumnDelimiter() {
		return csvColumnDelimiter;
	}

	public void setCsvColumnDelimiter(String csvColumnDelimiter) {
		this.csvColumnDelimiter = csvColumnDelimiter;
	}

	public MsFileVersion getMsFileVersion() {
		return msFileVersion;
	}

	public void setMsFileVersion(MsFileVersion msFileVersion) {
		this.msFileVersion = msFileVersion;
	}

	public Charset getCharset() {
		return charset;
	}

	public void setCharset(Charset charset) {
	}

	public CompressionType getCompressionType() {
		return compressionType;
	}

	public void setCompression(CompressionType compressionType) {
		this.compressionType = compressionType;
	}

}
