/**
 * Copyright (C) 2012  Dušan Vejnovič  <vaadin@dussan.org>
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 * http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */
package org.dussan.vaadin.dtable.windows.exporter.base;

import java.io.File;
import java.math.BigDecimal;
import java.math.BigInteger;
import java.sql.Time;
import java.sql.Timestamp;
import java.text.NumberFormat;
import java.text.SimpleDateFormat;
import java.util.ArrayList;
import java.util.Arrays;
import java.util.Collections;
import java.util.Date;
import java.util.List;
import java.util.Map;
import java.util.UUID;

import org.apache.commons.lang3.ArrayUtils;
import org.dussan.vaadin.dtable.DTable;
import org.dussan.vaadin.dtable.config.formatting.Formatting;
import org.dussan.vaadin.dtable.processors.CompressingProcessor;
import org.dussan.vaadin.dtable.processors.FormattingProcessor;
import org.dussan.vaadin.dtable.translations.TranslatedWord;

import com.github.wolfie.refresher.Refresher;
import com.github.wolfie.refresher.Refresher.RefreshListener;
import com.vaadin.terminal.FileResource;
import com.vaadin.ui.Alignment;
import com.vaadin.ui.Button;
import com.vaadin.ui.Button.ClickEvent;
import com.vaadin.ui.Button.ClickListener;
import com.vaadin.ui.HorizontalLayout;
import com.vaadin.ui.Label;
import com.vaadin.ui.VerticalLayout;
import com.vaadin.ui.Window;

@SuppressWarnings("serial")
public abstract class Exporter extends Window implements RefreshListener {

	public static final int ENABLE = 2000;
	public static final int DISABLE = 0;

	public static final int EXPORTING_SMALL_STEP = 500;
	public static final int EXPORTING_NORMAL_STEP = 1000;
	public static final int EXPORTING_LARGE_STEP = 50000;
	public static final int EXPORTING_EXTRA_LARGE_STEP = 100000;

	public static final float EXPORTING_NOT_STARTED = -1.0f;
	public static final float EXPORTING_BEGIN = 0.0f;
	public static final float EXPORTING_COMPLETE = 1.0f;
	public static final float EXPORTING_COMPLETED = 2.0f;
	public static final float EXPORTING_PREPARE = 3.0f;
	public static final float EXPORTING_PREPARED = 4.0f;
	public static final float EXPORTING_STOP = -98.0f;
	public static final float EXPORTING_STOPPED = -99.0f;
	public static final float EXPORTING_ERROR = -100.0f;

	private DTable dTable = null;
	private boolean isAttached = false;

	private boolean exportInterrupted = false;
	private float exportingStatus = EXPORTING_NOT_STARTED;
	private NumberFormat exportStatusFormatter = null;
	private ExportingType exportingType = null;
	private String tempFolder = null;
	private File archivedFile = null;

	private Button closeButton = null;
	private Button exportButton = null;
	private Refresher statusWatcher = null;
	private Label statusDescription = null;

	public Exporter(DTable dTable) {
		this.dTable = dTable;
		exportStatusFormatter = NumberFormat.getPercentInstance(getDTable()
				.getTableConfig().getLocale());
		exportStatusFormatter.setMinimumFractionDigits(2);
		exportStatusFormatter.setMaximumFractionDigits(2);
	}

	public class ExportProcess extends Thread {

		@Override
		public void run() {
			tempFolder = System.getProperty("java.io.tmpdir")
					+ File.separatorChar + UUID.randomUUID();

			int[] removedColumns = new int[] {};
			if (getDTable().getTableConfig().isEnabledAutoPrepareData()) {
				if (!getDTable().getTableConfig()
						.isEnabledAutoPrepareDataWithoutHiddenColumns()
						&& !getDTable().getTableProperties().isNoHiddenColumn()) {
					removedColumns = ArrayUtils
							.addAll(removedColumns, getDTable()
									.getTableProperties().getHiddenColumns());
				}
				if (!getDTable().getTableConfig()
						.isEnabledAutoPrepareDataWithoutRemovedColumns()
						&& !getDTable().getTableProperties()
								.isNoRemovedColumn()) {
					removedColumns = ArrayUtils.addAll(removedColumns,
							getDTable().getTableProperties()
									.getRemovedColumns());
				}
				Arrays.sort(removedColumns);
			}

			getDTable().getTableExporter()
					.checkBaseColumnsDefinition(
							getDTable().getTableProperties()
									.getCompleteColumnsHeader(),
							getDTable().getTableConfig()
									.getFormattingColumnsArray(),
							removedColumns);
			export();
		}

	}

	protected abstract boolean openFile(String name);

	protected abstract void writeToFile(String name, float completeStep,
			float step);

	protected abstract void closeFile();

	protected abstract void export();

	private static void deleteTempFolder(File folder) {
		try {
			if (folder.exists()) {
				File[] files = folder.listFiles();
				for (int i = 0; i < files.length; i++) {
					if (files[i].isDirectory()) {
						deleteTempFolder(files[i]);
					} else {
						files[i].delete();
					}
				}
			}
			folder.delete();
		} catch (Exception e) {
		}
	}

	private void closeWindow() {
		statusWatcher.setRefreshInterval(DISABLE);
		setExportingStatus(EXPORTING_NOT_STARTED);
		setVisible(false);
		getParent().removeWindow(this);
	}

	private void prepareExportedFile() {
		// get filenames to be added to archive
		List<File> archivedFiles = new ArrayList<File>();
		switch (getExportingType()) {
		case CSV:
			for (String name : getDTable().getTableExporter()
					.getColumnsDefinitionName()) {
				archivedFiles.add(getTempFile(name));
			}
			break;
		case MS_ACCESS:
		case MS_EXCEL:
		default:
			archivedFiles.add(getTempFile(getDTable().getTableExporter()
					.getFilename()));
			break;
		}
		// create archive
		try {
			float exportingStatus = 0f;
			archivedFile = new File(getExportedFile().getCanonicalPath()
					+ getDTable().getTableExporter().getCompressionType()
							.getExtension());
			switch (getDTable().getTableExporter().getCompressionType()) {
			case TAR_BZ2:
				exportingStatus = CompressingProcessor.tarBz2(archivedFile,
						archivedFiles);
				break;
			case TAR_GZ:
				exportingStatus = CompressingProcessor.tarGz(archivedFile,
						archivedFiles);
				break;
			case ZIP:
				exportingStatus = CompressingProcessor.zip(archivedFile,
						archivedFiles);
				break;
			case NONE:
			default:
				archivedFile = new File(getExportedFile().getCanonicalPath());
				exportingStatus = EXPORTING_PREPARED;
				break;
			}
			setExportingStatus(exportingStatus);
		} catch (Exception e) {
			archivedFile = null;
			setExportingStatus(EXPORTING_ERROR, e.getLocalizedMessage());
		}
	}

	private File getExportedFile() {
		try {
			if (!new File(tempFolder).exists()) {
				new File(tempFolder).mkdir();
			}
			if (getDTable().getTableExporter().isEnabledTimestamp()) {
				SimpleDateFormat timestampFormat = new SimpleDateFormat(
						getDTable().getTableExporter().getTimestampFormat());
				return (new File(tempFolder + File.separatorChar
						+ getDTable().getTableExporter().getFilename() + "."
						+ timestampFormat.format(new Date())
						+ exportingType.getExtension()));
			} else {
				return (new File(tempFolder + File.separatorChar
						+ getDTable().getTableExporter().getFilename()
						+ exportingType.getExtension()));
			}
		} catch (Exception e) {
			return null;
		}
	}

	private ExportingColumnType[] getDataType(String name) {
		Object[] rowData = getDTable().getExportedData(name, 0, 1);
		List<ExportingColumnType> dataTypes = new ArrayList<ExportingColumnType>();
		for (Object data : rowData) {
			if (data.getClass().equals(Boolean.class)) {
				dataTypes.add(ExportingColumnType.BOOLEAN);
			} else if (data.getClass().equals(BigDecimal.class)) {
				dataTypes.add(ExportingColumnType.BIG_DECIMAL);
			} else if (data.getClass().equals(BigInteger.class)) {
				dataTypes.add(ExportingColumnType.BIG_INTEGER);
			} else if (data.getClass().equals(Date.class)
					|| data.getClass().equals(Time.class)
					|| data.getClass().equals(Timestamp.class)) {
				dataTypes.add(ExportingColumnType.DATE);
			} else if (data.getClass().equals(Double.class)) {
				dataTypes.add(ExportingColumnType.DOUBLE);
			} else if (data.getClass().equals(Integer.class)) {
				dataTypes.add(ExportingColumnType.INT);
			} else if (data.getClass().equals(Float.class)) {
				dataTypes.add(ExportingColumnType.FLOAT);
			} else if (data.getClass().equals(Long.class)
					|| data.getClass().equals(Short.class)) {
				dataTypes.add(ExportingColumnType.LONG);
			} else if (data.getClass().equals(String.class)) {
				dataTypes.add(ExportingColumnType.TEXT);
			} else {
				dataTypes.add(ExportingColumnType.MEMO);
			}
		}
		return dataTypes.toArray(new ExportingColumnType[] {});
	}

	protected boolean canContinue() {
		return (getExportingStatus() >= EXPORTING_STOP && !isExportInterrupted());
	}

	/*
	 * Object[] definition: 0: column header name, 1: formatting, 2: exported
	 * column type, 3: length or precision
	 */
	protected boolean prepareExportedData(String name) {
		if (getDTable().prepareExportedData(name)) {
			ExportingColumnType[] columnsDataType = getDataType(name);
			Map<Integer, Object[]> definitions = getDTable().getTableExporter()
					.getColumnDefinition(name);

			List<Integer> columns = new ArrayList<Integer>(definitions.keySet());
			Collections.sort(columns);
			for (int column = 0; column < columns.size(); column++) {
				String header = getDTable().getTableExporter()
						.getColumnDefinitionHeader(name, columns.get(column));
				Formatting formatting = getDTable().getTableExporter()
						.getColumnDefinitionFormatting(name,
								columns.get(column));
				ExportingColumnType columnExportedType = getDTable()
						.getTableExporter().getColumnDefinitionExportingType(
								name, columns.get(column));
				Integer lengthOrPrecision = getDTable().getTableExporter()
						.getColumnDefinitionLengthOrPrecision(name,
								columns.get(column));

				// check column name
				if (header == null || header.isEmpty()) {
					header = "HEADER " + columns.get(column);
				}
				// check data type
				switch (getExportingType()) {
				case MS_ACCESS:
					if (columnExportedType == null) {
						if (ExportingColumnType.MEMO
								.equals(columnsDataType[column])
								|| ExportingColumnType.TEXT
										.equals(columnsDataType[column])) {
							columnExportedType = ExportingColumnType.MEMO;
						} else {
							columnExportedType = columnsDataType[column];
						}
					} else if (ExportingColumnType.TEXT
							.equals(columnExportedType)
							&& lengthOrPrecision == null) {
						columnExportedType = ExportingColumnType.MEMO;
					} else if (columnExportedType != null
							&& !columnsDataType[column]
									.equals(columnExportedType)
							&& !ExportingColumnType.MEMO
									.equals(columnExportedType)
							&& !ExportingColumnType.TEXT
									.equals(columnExportedType)) {
						setExportingStatus(
								EXPORTING_ERROR,
								getDTable()
										.getTranslation()
										.get(getDTable().getTableConfig()
												.getLocale(),
												TranslatedWord.EXPORTED_COLUMN_DATA_TYPE_AND_COLUMN_DEFINITION_TYPE_ARE_NOT_THE_SAME));
						return false;
					}
					break;
				default:
					break;
				}
				definitions.put(columns.get(column), new Object[] { header,
						formatting, columnExportedType, lengthOrPrecision });
			}

			// write back to original
			getDTable().getTableExporter().setColumnDefinition(name,
					definitions);
			return true;
		} else {
			return false;
		}
	}

	protected long getAllRows(String name) {
		return getDTable().getExportedAllRows(name);
	}

	protected int getAllColumns(String name) {
		return getDTable().getTableExporter().getColumnDefinition(name).size();
	}

	protected File getTempFile(String name) {
		try {
			if (!new File(tempFolder).exists()) {
				new File(tempFolder).mkdir();
			}
			return (new File(tempFolder + File.separatorChar + name
					+ exportingType.getExtension()));
		} catch (Exception e) {
			return null;
		}
	}

	protected String getCharset() {
		return getDTable().getTableExporter().getCharset().getCharset();
	}

	protected Object[] getData(String name, int startRow, int rows) {
		int allColumns = getAllColumns(name);
		Object[] data = getDTable().getExportedData(name, startRow, rows);
		if (data != null && data.length > 0
				&& !ExportingType.MS_ACCESS.equals(getExportingType())
				&& getDTable().getTableExporter().isEnabledFormattedData()) {
			for (int row = 0; row < rows; row++) {
				List<Integer> columns = new ArrayList<Integer>(getDTable()
						.getTableExporter().getColumnDefinition(name).keySet());
				Collections.sort(columns);
				for (int column = 0; column < allColumns; column++) {
					Formatting formatting = getDTable().getTableExporter()
							.getColumnDefinitionFormatting(name,
									columns.get(column));
					data[row * allColumns + column] = FormattingProcessor
							.process(formatting,
									data[row * allColumns + column]);
				}
			}
		}
		return Arrays.copyOfRange(data, 0, rows * allColumns);
	}

	protected DTable getDTable() {
		return dTable;
	}

	protected boolean isExportInterrupted() {
		return exportInterrupted;
	}

	protected float getExportingStatus() {
		return exportingStatus;
	}

	protected void setExportingStatus(float exportingStatus, String description) {
		setExportingStatus(exportingStatus);
		if (description != null && !description.isEmpty()) {
			getApplication().getMainWindow().showNotification(
					getDTable().getTranslation().get(
							getDTable().getTableConfig().getLocale(),
							TranslatedWord.ERROR), description,
					Notification.TYPE_ERROR_MESSAGE);
		}
	}

	protected void setExportingStatus(float exportingStatus) {
		if (canContinue()) {
			this.exportingStatus = exportingStatus;
		} else if (EXPORTING_STOP < getExportingStatus()) {
			this.exportingStatus = EXPORTING_STOP;
		} else {
			this.exportingStatus = exportingStatus;
		}

		if (this.exportingStatus == EXPORTING_ERROR
				|| this.exportingStatus == EXPORTING_STOPPED
				|| this.exportingStatus == EXPORTING_COMPLETED) {
			getDTable().exportProcessCompleted();
		}
	}

	protected ExportingType getExportingType() {
		return exportingType;
	}

	protected void setExportingType(ExportingType exportingType) {
		this.exportingType = exportingType;
	}

	@Override
	public void attach() {
		if (!isAttached) {
			VerticalLayout windowLayout = (VerticalLayout) getContent();
			windowLayout.addStyleName("v-dtable-dialog");
			windowLayout.setMargin(false);
			windowLayout.setSpacing(false);

			center();
			setClosable(false);
			setDraggable(false);
			setResizable(false);
			setModal(true);
			setWidth(350, UNITS_PIXELS);

			HorizontalLayout layout = new HorizontalLayout();
			layout.addStyleName("context");
			layout.setWidth(100, UNITS_PERCENTAGE);
			layout.setMargin(true);
			layout.setSpacing(true);
			windowLayout.addComponent(layout);

			statusDescription = new Label(getDTable().getTranslation().get(
					getDTable().getTableConfig().getLocale(),
					TranslatedWord.TO_EXPORT_THE_DATA_PRESS_THE_BUTTON_EXPORT));
			layout.addComponent(statusDescription);

			HorizontalLayout space1 = new HorizontalLayout();
			space1.setWidth(100, UNITS_PERCENTAGE);
			space1.setHeight(6, UNITS_PIXELS);
			windowLayout.addComponent(space1);
			windowLayout.setComponentAlignment(space1, Alignment.BOTTOM_CENTER);

			HorizontalLayout buttonLayout = new HorizontalLayout();
			buttonLayout.setWidth(100, UNITS_PERCENTAGE);
			buttonLayout.setMargin(false);
			buttonLayout.setSpacing(true);
			windowLayout.addComponent(buttonLayout);
			windowLayout.setComponentAlignment(buttonLayout,
					Alignment.BOTTOM_CENTER);

			HorizontalLayout space2 = new HorizontalLayout();
			space2.setWidth(100, UNITS_PERCENTAGE);
			space2.setHeight(6, UNITS_PIXELS);
			windowLayout.addComponent(space2);
			windowLayout.setComponentAlignment(space2, Alignment.BOTTOM_CENTER);

			exportButton = new Button(getDTable().getTranslation().get(
					getDTable().getTableConfig().getLocale(),
					TranslatedWord.EXPORT), new ClickListener() {
				@Override
				public void buttonClick(ClickEvent event) {
					// export
					if (getDTable()
							.getTranslation()
							.get(getDTable().getTableConfig().getLocale(),
									TranslatedWord.EXPORT)
							.equals(exportButton.getCaption())) {
						statusDescription.setValue(getDTable().getTranslation()
								.get(getDTable().getTableConfig().getLocale(),
										TranslatedWord.PREPARING_DATA)
								+ "...");
						exportButton.setCaption(getDTable().getTranslation()
								.get(getDTable().getTableConfig().getLocale(),
										TranslatedWord.STOP_PREPARING_DATA));
						closeButton.setEnabled(false);
						statusWatcher.setRefreshInterval(ENABLE);
						new ExportProcess().start();
					} // stop preparing data
					else if (getDTable()
							.getTranslation()
							.get(getDTable().getTableConfig().getLocale(),
									TranslatedWord.STOP_PREPARING_DATA)
							.equals(exportButton.getCaption())) {
						exportInterrupted = true;
						exportButton.setEnabled(false);
					} // download the data
					else if (getDTable()
							.getTranslation()
							.get(getDTable().getTableConfig().getLocale(),
									TranslatedWord.DOWNLOAD_THE_DATA)
							.equals(exportButton.getCaption())) {
						FileResource fileResource = new FileResource(
								archivedFile, getApplication());
						getApplication().getMainWindow().open(fileResource,
								"_top");
						closeWindow();
					}
				}
			});
			buttonLayout.addComponent(exportButton);
			buttonLayout.setComponentAlignment(exportButton,
					Alignment.MIDDLE_RIGHT);

			closeButton = new Button(getDTable().getTranslation().get(
					getDTable().getTableConfig().getLocale(),
					TranslatedWord.CLOSE), new ClickListener() {
				@Override
				public void buttonClick(ClickEvent event) {
					if (tempFolder != null) {
						deleteTempFolder(new File(tempFolder));
					}
					closeWindow();
				}
			});
			buttonLayout.addComponent(closeButton);
			buttonLayout.setComponentAlignment(closeButton,
					Alignment.MIDDLE_LEFT);

			statusWatcher = new Refresher();
			statusWatcher.setRefreshInterval(500);
			statusWatcher.addListener((RefreshListener) this);
			windowLayout.addComponent(statusWatcher);

			isAttached = true;
		}
		super.attach();
	}

	@Override
	public void detach() {
		if (isAttached) {
			statusWatcher.removeListener((RefreshListener) this);
			removeAllComponents();
			isAttached = false;
		}
		super.detach();
	}

	@Override
	public void refresh(Refresher source) {
		if (getExportingStatus() == EXPORTING_ERROR) {
			statusWatcher.setRefreshInterval(DISABLE);
			statusDescription.setValue(getDTable().getTranslation().get(
					getDTable().getTableConfig().getLocale(),
					TranslatedWord.IN_PREPARING_DATA_AN_ERROR_OCCURRED));
			exportButton.setVisible(false);
			closeButton.setEnabled(true);
		} else if (getExportingStatus() == EXPORTING_STOP) {
			setExportingStatus(EXPORTING_STOPPED);
		} else if (getExportingStatus() == EXPORTING_STOPPED) {
			statusWatcher.setRefreshInterval(DISABLE);
			statusDescription.setValue(getDTable().getTranslation().get(
					getDTable().getTableConfig().getLocale(),
					TranslatedWord.PREPARING_DATA_IS_STOPPED));
			exportButton.setVisible(false);
			closeButton.setEnabled(true);
		} else if (getExportingStatus() >= EXPORTING_BEGIN
				&& getExportingStatus() <= EXPORTING_COMPLETE) {
			statusDescription.setValue(getDTable().getTranslation().get(
					getDTable().getTableConfig().getLocale(),
					TranslatedWord.PREPARING_DATA)
					+ " ("
					+ exportStatusFormatter.format(exportingStatus)
					+ ")...");
			if (getExportingStatus() == EXPORTING_COMPLETE) {
				statusDescription.setValue(getDTable().getTranslation().get(
						getDTable().getTableConfig().getLocale(),
						TranslatedWord.PREPARING_DATA_IS_COMPLETED));
				setExportingStatus(EXPORTING_COMPLETED);
				exportButton.setEnabled(false);
			}
		} else if (getExportingStatus() == EXPORTING_COMPLETED) {
			statusDescription.setValue(getDTable().getTranslation().get(
					getDTable().getTableConfig().getLocale(),
					TranslatedWord.PREPARING_DATA_FOR_DOWNLOAD));
			setExportingStatus(EXPORTING_PREPARE);
		} else if (getExportingStatus() == EXPORTING_PREPARE) {
			prepareExportedFile();
		} else if (getExportingStatus() == EXPORTING_PREPARED) {
			statusWatcher.setRefreshInterval(DISABLE);
			statusDescription.setValue(getDTable().getTranslation().get(
					getDTable().getTableConfig().getLocale(),
					TranslatedWord.DATA_ARE_READY_FOR_DOWNLOAD));
			exportButton.setCaption(getDTable().getTranslation().get(
					getDTable().getTableConfig().getLocale(),
					TranslatedWord.DOWNLOAD_THE_DATA));
			exportButton.setEnabled(true);
			closeButton.setEnabled(true);
		}
	}

}
